@@ -3,7 +3,7 @@ from __future__ import division
 from __future__ import print_function
 import threading
 import inspect
-from inspect import _empty, signature
+from inspect import  signature
 import collections
 import os
 from enum import Enum, unique
@@ -22,29 +22,38 @@ import time
 import types
 import traceback
 import json
-__all__ = ['get_session','get_trident_dir','epsilon','set_epsilon','floatx','set_floatx','if_else','camel2snake','snake2camel','to_onehot','to_list','addindent','format_time', 'get_time_suffix', 'get_function', 'get_class', 'get_terminal_size', 'gcd', 'get_divisors', 'isprime', 'next_prime', 'prev_prime', 'nearest_prime','PrintException','unpack_singleton','enforce_singleton','OrderedDict','get_python_function_arguments','map_function_arguments','ClassfierType','PaddingMode', 'DataRole',
-
-           'ExpectDataType', 'GetImageMode', 'make_dir_if_need', 'ShortcutMode', 'DataSpec','get_argument_maps','get_args_spec']
-
-
+from typing import List, Set, Tuple, Dict
 
+__all__ = ['get_session','get_trident_dir','get_signature','epsilon','set_epsilon','floatx','set_floatx','if_else','camel2snake','snake2camel','to_onehot','to_list','addindent','format_time', 'get_time_suffix', 'get_function', 'get_class', 'get_terminal_size', 'gcd', 'get_divisors', 'isprime', 'next_prime', 'prev_prime', 'nearest_prime','PrintException','unpack_singleton','enforce_singleton','OrderedDict','get_python_function_arguments','map_function_arguments','ClassfierType','PaddingMode', 'DataRole',
+
+           'ExpectDataType', 'GetImageMode','split_path', 'make_dir_if_need','sanitize_path', 'ShortcutMode', 'DataSpec','get_argument_maps','get_args_spec']
+
+def sanitize_path(path):
+    if isinstance(path,str) :
+        return os.path.normpath(path.strip()).replace('\\','/')
+    else:
+        return path
 
 
 
+_trident_dir=''
 if 'TRIDENT_HOME' in os.environ:
     _trident_dir = os.environ.get('TRIDENT_HOME')
-    if not os.path.exists(_trident_dir):
-        try:
-            os.makedirs(_trident_dir)
-        except OSError:
-            # Except permission denied and potential race conditions
-            # in multi-threaded environments.
-            pass
 else:
     _trident_base_dir = os.path.expanduser('~')
     if not os.access(_trident_base_dir, os.W_OK):
-        _trident_base_dir = '/tmp'
-    _trident_dir = os.path.join(_trident_base_dir, '.trident')
+        _trident_dir = '/tmp/.trident'
+    else:
+        _trident_dir = os.path.expanduser('~/.trident')
+
+_trident_dir=sanitize_path(_trident_dir)
+if not os.path.exists(_trident_dir):
+    try:
+        os.makedirs(_trident_dir)
+    except OSError:
+        # Except permission denied and potential race conditions
+        # in multi-threaded environments.
+        pass
 
 
 # Attempt to read Trident config file.
@@ -63,20 +72,25 @@ _SESSION.epsilon= 1e-8
 _SESSION.numpy_print_format='{0:.4e}'
 
 
+_SESSION.user_defined_device=None
 
 
-_config_path = os.path.expanduser(os.path.join(_trident_dir, 'trident.json'))
+
+
+_config_path = os.path.expanduser('~/.trident/trident.json')
 if os.path.exists(_config_path):
+    _config = {}
     try:
         with open(_config_path) as f:
             _config = json.load(f)
     except ValueError:
-        _config = {}
+        pass
     for k,v in _config.items():
         try:
             if k=='floatx':
                 assert v in {'float16', 'float32', 'float64'}
-            _SESSION.__setattr__(k,v)
+            if k!='trident_dir':
+                _SESSION.__setattr__(k,v)
         except :
             exc_type, exc_obj, tb = sys.exc_info()
             traceback.print_exception(exc_type, exc_obj, tb, limit=2, file=sys.stdout)
@@ -107,6 +121,7 @@ class IntevalUnit(Enum):
     epoch = 'epoch'
     once = 'once'
 
+
 np.set_printoptions(formatter={'float_kind':lambda x: _SESSION.numpy_print_format.format(x)})
 
 
@@ -122,8 +137,13 @@ def _is_c_contiguous(data):
 def get_session():
     return _SESSION
 
+
+def set_session(key,value):
+    setattr(_SESSION,key,value)
+    return _SESSION
+
 def get_trident_dir():
-    return _trident_dir
+    return _SESSION.trident_dir
 
 
 
@@ -137,7 +157,17 @@ def PrintException():
     line = linecache.getline(filename, lineno, f.f_globals)
     print('EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj))
     traceback.print_tb(tb, limit=1, file=sys.stdout)
-    traceback.print_exception(exc_type, exc_obj, tb, limit=3, file=sys.stdout)
+    traceback.print_exception(exc_type, exc_obj, tb, limit=2, file=sys.stdout)
+
+
+
+def get_signature(fn,skip_default=False):
+    signature = OrderedDict()
+    parameters = list(inspect.signature(fn).parameters.values())
+    for k in parameters:
+        if skip_default==False or (skip_default==True and k.default ==inspect._empty):
+            signature[k.name]= k.default if k.default !=inspect._empty else None
+    return signature
 
 
 
@@ -222,6 +252,8 @@ def to_list(x):
         return [x[i] for i in range(len(x))]
     elif isinstance(x,np.ndarray):
         return x.tolist()
+    elif hasattr(x, 'tolist')and callable(x.tolist):
+        return x.tolist()
     elif isinstance(x,(int,float)):
         return [x]
     elif isinstance(x,type({}.keys())):
@@ -230,8 +262,10 @@ def to_list(x):
         return list(x)
     elif isinstance(x,types.GeneratorType):
         return list(x)
-    elif hasattr(x, 'tolist'):
-        return x.tolist()
+    elif inspect.isgenerator(x):
+        return list(x)
+    elif isinstance(x, collections.Iterable):
+        return list(x)
     else:
         try:
             return list(x)
@@ -548,22 +582,33 @@ def map_function_arguments(params, params_dict, *args, **kwargs):
 # def map_if_possible(obj_to_map, *args, **kwargs):
 #     if inspect.isfunction(obj_to_map):
 
+
+
+def split_path(path):
+    if path is None or len(path)==0:
+        return '','',''
+    path=sanitize_path(path)
+    folder,filename=os.path.split(path)
+    ext=''
+    if '.' in filename:
+        filename, ext = os.path.splitext(filename)
+        filename, ext2 = os.path.splitext(filename)
+        ext = ext2 + ext
+    else:
+        folder = os.path.join(folder, filename)
+        filename = ''
+    return folder,filename,ext
+
+
 def make_dir_if_need(path):
-    path=os.path.normpath(path)
-    folder,file=os.path.split(path)
-    if folder=='':
-        folder, file = os.path.split(file)
-    if '.' not in file and len(file)>=1:
-        folder=os.path.join(folder,file)
-        file=''
+    folder,filename,ext=split_path(path)
     if len(folder)>0 and not os.path.exists(folder):
         try:
             os.makedirs(folder)
         except Exception as e:
             PrintException()
             sys.stderr.write('folder:{0} is not valid path'.format(folder))
-    return os.path.join(folder,file)
-
+    return sanitize_path(path)
 
 class OrderedDict(collections.OrderedDict):
     def __init__(self):
