@@ -1,7 +1,14 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-import matplotlib
+from ..misc.ipython_utils import is_in_ipython,is_in_colab
+if is_in_ipython():
+    from IPython import display
+if not is_in_colab:
+    import matplotlib
+    matplotlib.use('Qt5Agg' if not is_in_ipython() else 'TkAgg')
+else:
+    import matplotlib
 import matplotlib.pyplot as plt
 import os
 import sys
