@@ -28,10 +28,10 @@ import torch.onnx
 import torchvision
 
 import numpy as np
-from .common import to_list,addindent,camel2snake,snake2camel,unpack_singleton,enforce_singleton,OrderedDict
+from .common import to_list,addindent,camel2snake,snake2camel,unpack_singleton,enforce_singleton,OrderedDict,get_signature
 from .pytorch_ops import *
 
-__all__ = ['get_uid','to_numpy','to_tensor','print_network','plot_tensor_grid','summary','calculate_flops', 'Layer', 'Sequential', 'Input', 'get_device', 'load','Combine','ReplayBuffer','check_keys','try_map_args_and_call','get_signature']
+__all__ = ['get_uid','to_numpy','to_tensor','print_network','plot_tensor_grid','summary','calculate_flops', 'Layer', 'Sequential', 'Input', 'get_device', 'load','Combine','ReplayBuffer','check_keys','try_map_args_and_call']
 
 version=torch.__version__
 sys.stderr.write('Pytorch version:{0}.\n'.format(version))
@@ -47,12 +47,9 @@ if torch.cuda.is_available() :
 def get_device():
     return  _device
 
-def load(model,path):
-    checkpoint = torch.load(path)
-    model.load_state_dict(checkpoint['state_dict'])
-    return model
-
 
+def load(path):
+    return torch.load(path).to(_device)
 
 import sys
 
@@ -627,7 +624,7 @@ class Sequential(Layer):
                 self.add_module(key, module)
         elif len(args) == 1 and isinstance(args[0], (list,nn.ModuleList)):
             for  idx, module in enumerate(args[0]):
-                self.add_module(str(idx), module[1])
+                self.add_module(str(idx), module)
         else:
             for idx, module in enumerate(args):
                 self.add_module(str(idx), module)
@@ -659,10 +656,8 @@ class Sequential(Layer):
             raise KeyError("module name can't contain \".\"")
         elif name == '':
             raise KeyError("module name can't be empty string \"\"")
-        if len(self._modules) > 0 and self._modules[list(self._modules.keys())[0]]._input_shape is not None and \
-                self._modules[list(self._modules.keys())[-1]].built and self._modules[
-            list(self._modules.keys())[-1]]._output_shape is not None:
-            module.input_shape = self._modules[list(self._modules.keys())[-1]]._output_shape
+        if len(self._modules) > 0 and self._input_shape is not None and self[-1].built and self[-1]._output_shape is not None:
+            module.input_shape = self[-1]._output_shape
         self._modules[name] = module
         self._output_shape = module._output_shape
 
@@ -675,6 +670,9 @@ class Sequential(Layer):
             input_shape.insert(0, 2)
             data = torch.tensor(np.random.standard_normal(input_shape))
             out = self(data)
+    def remove_at(self,idx):
+        self.__delitem__(idx)
+
 
 
     def _get_item_by_idx(self, iterator, idx):
@@ -688,7 +686,10 @@ class Sequential(Layer):
 
     def __getitem__(self, idx):
         if isinstance(idx, slice):
-            return self.__class__(list(self._modules.items())[idx])
+            returnDict=OrderedDict()
+            for k, v in list(self._modules.items())[idx]:
+                returnDict[k] = v
+            return returnDict
         else:
             return self._get_item_by_idx(self._modules.values(), idx)
 
@@ -954,10 +955,10 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
         input_size = [input_size]
 
     if isinstance(input_size, int):
-        x = [torch.rand(2, input_size).type(dtype)]
+        x = [torch.rand(2, input_size).type(dtype).to("cuda" if model.weights[0].data.is_cuda else "cpu")]
     else:
         # batch_size of 2 for batchnorm
-        x = [torch.rand(2, *in_size).type(dtype) for in_size in input_size]
+        x = [torch.rand(2, *in_size).type(dtype).to("cuda" if model.weights[0].data.is_cuda else "cpu") for in_size in input_size]
     # p    rint(type(x[0]))
 
     # create properties
@@ -974,8 +975,6 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
     # remove these hooks
     for h in hooks:
         h.remove()
-    layer_width=(max([len(layer) for layer in summary])//5+1)*5
-
 
     print("--------------------------------------------------------------------------------------------------------------------------------")
     line_new = "{0:^40s} {1:^20s}  {2:^20s} {3:^8s}  {4:^8s}  {5:^12s}".format("Layer (type)", "Output Shape","Weight ","Bias", "Param #", "FLOPS #")
@@ -1352,14 +1351,6 @@ def check_keys(model, pretrained_state_dict):
     return True
 
 
-def get_signature(fn,skip_default=False):
-    signature = OrderedDict()
-    parameters = list(inspect.signature(fn).parameters.values())
-    for k in parameters:
-        if skip_default==False or (skip_default==True and k.default ==inspect._empty):
-            signature[k.name]= k.default if k.default !=inspect._empty else None
-    return signature
-
 
 def try_map_args_and_call(fn, data: OrderedDict,data_feed=None,):
     if isinstance(fn,torch.Tensor):
