@@ -10,11 +10,37 @@ from functools import partial
 import numpy as np
 from .common import *
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
-__all__ = ['reshape','argmax','gram_matrix','log_sum_exp','reduce_max','reduce_min','reduce_mean','reduce_sum','expand_dims','shuffle','random_choice','meshgrid','element_cosine_distance','gram_matrix','get_rotation_matrix2d','warp_affine','binary_crossentropy']
+__all__ = ['is_nan','is_abnormal_number','reshape','argmax','gram_matrix','log_sum_exp','reduce_max','reduce_min','reduce_mean','reduce_sum','expand_dims','shuffle','random_choice','meshgrid','element_cosine_distance','gram_matrix','get_rotation_matrix2d','warp_affine','binary_crossentropy']
 
 
 
 
+def is_nan(x):
+    if isinstance(x,torch.Tensor):
+        return torch.isnan(x).any()
+    elif isinstance(x,nn.Module):
+        for para in x.parameters():
+            if torch.isnan(para).any():
+                return True
+        return False
+    elif isinstance(x, np.ndarray):
+        return np.isnan(x).any()
+    else:
+        raise NotImplementedError
+
+def is_abnormal_number(x):
+    if isinstance(x,torch.Tensor):
+        return torch.isnan(x).any() or torch.isinf(x).any()or torch.isinf(-x).any()
+    elif isinstance(x,nn.Module):
+        for para in x.parameters():
+            if torch.isnan(para).any() or torch.isinf(para).any()or torch.isinf(-para).any():
+                return True
+        return False
+    elif isinstance(x, np.ndarray):
+        return np.isnan(x).any() or np.isinf(x).any() or np.isneginf(x).any()
+    else:
+        raise NotImplementedError
+
 
 
 
