@@ -2,8 +2,8 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 import sys
+import inspect
 import numpy as np
-from collections import OrderedDict
 import random
 from itertools import islice
 import tensorflow as tf
@@ -16,9 +16,9 @@ from tensorflow.python.util import object_identity
 import copy
 
 from tensorflow.python.keras import layers as layer_module
-from tensorflow.python.keras.engine import base_layer
 from tensorflow.python.keras.engine import base_layer_utils
 from tensorflow.python.keras.engine import input_layer
+from tensorflow.python.keras.engine.base_layer import Layer
 from tensorflow.python.keras.engine import training
 from tensorflow.python.keras.engine import training_utils
 from tensorflow.python.keras.saving.saved_model import model_serialization
@@ -30,14 +30,15 @@ from tensorflow.python.training.tracking import layer_utils as trackable_layer_u
 from tensorflow.python.util import nest
 from tensorflow.python.util import tf_inspect
 from tensorflow.python.util.tf_export import keras_export
+from tensorflow_core.python.framework.ops import EagerTensor
 
 from ..layers.tensorflow_layers import *
 from ..layers.tensorflow_normalizations import *
 from ..layers.tensorflow_activations import *
 from ..layers.tensorflow_normalizations import *
 from ..data.tensorflow_datasets import *
-from ..backend.common import floatx, addindent, get_time_suffix, format_time, get_terminal_size, snake2camel, \
-    PrintException, to_list, unpack_singleton, enforce_singleton
+from .common import floatx, addindent, OrderedDict,get_signature,get_time_suffix, format_time, get_terminal_size, snake2camel, \
+    PrintException, to_list, unpack_singleton, enforce_singleton,OrderedDict,get_signature
 
 __all__ = ['register_keras_custom_object', 'to_numpy', 'to_tensor', 'get_flops', 'Input', 'Sequential',
            'ConcatContainer', 'ReplayBuffer', 'print_summary']
@@ -48,6 +49,7 @@ sys.stderr.write('Tensorflow version:{0}.\n'.format(version.VERSION))
 if version.VERSION < '2.0.0':
     raise ValueError('Not support Tensorflow below 2.0')
 
+tf.executing_eagerly()
 
 gpus = tf.config.experimental.list_physical_devices('GPU')
 if gpus:
@@ -752,3 +754,52 @@ def print_summary(model, line_length=None, positions=None, print_fn=None):
     print_fn('Trainable params: {:,}'.format(trainable_count))
     print_fn('Non-trainable params: {:,}'.format(non_trainable_count))
     print_fn('_' * line_length)
+
+
+
+
+
+
+
+
+def try_map_args_and_call(fn, data: OrderedDict,data_feed=None,):
+    if isinstance(fn,(tf.Tensor,EagerTensor)):
+        return fn
+    else:
+        arg_map = OrderedDict()
+        if isinstance(fn,Layer) :
+            for arg in fn.signature.key_list:
+                if arg in data_feed:
+                    arg_map[arg]=data[data_feed[arg]]
+                else:
+                    raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
+            #print('arg_map',arg_map.key_list)
+            out=fn(*arg_map.value_list)
+            return out
+        elif hasattr(fn,'signature') and callable(fn):
+            for arg in fn.signature.key_list:
+                if arg in data:
+                    arg_map[arg]=data[arg]
+                elif arg in data_feed:
+                    arg_map[arg]=data[data_feed[arg]]
+                else:
+
+                    raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
+            #print('arg_map', arg_map.key_list)
+            out=fn(*arg_map.value_list)
+            return out
+        elif  callable(fn):
+
+            args=get_signature(fn).key_list
+            for arg in args:
+                if arg in  data_feed:
+                    arg_map[arg]=data[data_feed[arg]]
+                else:
+                    arg_map[arg] = ''
+            #print('arg_map', arg_map.key_list)
+            out = fn(*arg_map.value_list)
+            return out
+        else:
+
+            print('uncomplete arg_map', arg_map.key_list)
+
