@@ -39,8 +39,9 @@ class MixupCallback(RegularizationCallbacksBase):
         self.loss_weight=loss_weight
     def on_loss_calculation_end(self, training_context):
         '''Returns mixed inputs, pairs of targets, and lambda'''
-        x=to_numpy(training_context['current_input'])
-        y=to_numpy(training_context['current_target'])
+        train_data = training_context['train_data']
+        x = to_numpy(train_data.value_list[0])
+        y = to_numpy(train_data.value_list[1])
         model=training_context['current_model']
 
 
@@ -62,7 +63,7 @@ class MixupCallback(RegularizationCallbacksBase):
                 array2image(item).save('Results/mixup_{0}.jpg'.format(get_time_suffix()))
 
         y_a, y_b = y, y[index]
-        pred=model(to_tensor(mixed_x))
+        pred=model(to_tensor(mixed_x,requires_grad=True))
         this_loss=lam * self.loss_criterion(pred, to_tensor(y_a)) + (1 - lam) * self.loss_criterion(pred,to_tensor(y_b))
         if 'mixup_loss' not in training_context['losses']:
             training_context['losses']['mixup_loss'] = []
@@ -98,8 +99,9 @@ class CutMixCallback(RegularizationCallbacksBase):
 
     def on_loss_calculation_end(self, training_context):
         '''Returns mixed inputs, pairs of targets, and lambda'''
-        x = to_numpy(training_context['current_input'])
-        y = to_numpy(training_context['current_target'])
+        train_data = training_context['train_data']
+        x = to_numpy(train_data.value_list[0])
+        y = to_numpy(train_data.value_list[1])
         model = training_context['current_model']
         if self.alpha > 0:
             lam = np.random.beta(self.alpha, self.alpha)
@@ -121,7 +123,7 @@ class CutMixCallback(RegularizationCallbacksBase):
                 item = unnormalize(0, 255)(item)
                 array2image(item).save('Results/cutmix_{0}.jpg'.format(get_time_suffix()))
 
-        pred = model(to_tensor(x))
+        pred = model(to_tensor(x,requires_grad=True))
         this_loss = lam * self.loss_criterion(pred, to_tensor(y_a)) + (1 - lam) * self.loss_criterion(pred, to_tensor(y_b))
         if 'mixup_loss' not in training_context['losses']:
             training_context['losses']['cutmix_loss'] = []
@@ -159,3 +161,4 @@ class GradientClippingCallback(RegularizationCallbacksBase):
 
 
 
+
