@@ -5,7 +5,7 @@ import os
 import sys
 from ..callbacks import CallbackBase
 from ..backend.common import *
-from ..backend.load_backend import get_backend
+from ..backend.load_backend import *
 from ..backend.pillow_backend import image2array
 from ..misc.ipython_utils import is_in_ipython, is_in_colab
 from ..data.mask_common import label2color
@@ -18,13 +18,7 @@ if is_in_ipython() or is_in_colab():
 _session = get_session()
 _backend = get_backend()
 
-if get_backend() == 'pytorch':
-    from ..backend.pytorch_backend import to_numpy, to_tensor
-    from ..backend.pytorch_ops import *
-elif get_backend() == 'tensorflow':
-    from ..backend.tensorflow_backend import to_numpy, to_tensor
-elif get_backend() == 'cntk':
-    from ..backend.cntk_backend import to_numpy, to_tensor
+
 
 __all__ = ['VisualizationCallbackBase', 'TileImageCallback', 'PrintGradientsCallback', 'SegTileImageCallback',
            'PlotLossMetricsCallback']
@@ -72,20 +66,28 @@ class TileImageCallback(VisualizationCallbackBase):
         data = training_context['train_data']
         model = training_context['current_model']
 
-        if len(data_feed) == len(data) :
-            input = data.value_list[0]
-            target =data.value_list[1]
-            output =data.value_list[2]
+        if len(data_feed) ==3 and  len(data)<=3 :
+            input = data[data_feed.value_list[0]]
+            output = data[data_feed.value_list[1]]
+            target =data[data_feed.value_list[2]]
+            #loss function signature  loss(output,target)
+
         else:
             for data_key in data.key_list:
                 if data_key == data_feed[model.signature.key_list[0]]:
                     input = data[data_feed[model.signature.key_list[0]]]
-                elif 'output' in data_key and not 'target' in data_key and data_key not in data_feed.value_list:
+                elif data_key=='output':
+                    output = data['output']
+                elif data_key=='target':
+                    output = data['target']
+                elif 'output' in data_key or 'pred' in data_key:
                         output = data[data_key]
-                elif ('target' in data_key or 'label' in data_key or 'mask' in data_key) and not 'output' in data_key and data_key in data_feed.value_list:
+                elif ('target' in data_key or 'label' in data_key or 'mask' in data_key) :
                     target = data[data_key]
-            if output is None and 'tensor' in model.__class__.__name__.lower():
-                output=model.clone()
+        if output is None and 'tensor' in model.__class__.__name__.lower():
+            output=model.clone()
+        elif output is None and input is not None :
+            output = model(input)
 
         if self.include_input and input is not None:
             if self.reverse_image_transform is not None:
@@ -120,11 +122,11 @@ class TileImageCallback(VisualizationCallbackBase):
         tile_rgb_images(*tile_images_list,row=self.row, save_path=os.path.join(self.save_path, self.tile_image_name_prefix),imshow=True)
 
     def on_batch_end(self, training_context):
-        if self.batch_inteval > 0 and (training_context['current_batch'] + 1) % self.batch_inteval == 0:
+        if self.batch_inteval > 0 and (training_context['current_batch']  % self.batch_inteval == 0):
             self.plot_tile_image(training_context)
 
     def on_epoch_end(self, training_context):
-        if self.epoch_inteval > 0 and (training_context['current_epoch'] + 1) % self.epoch_inteval == 0:
+        if self.epoch_inteval > 0 and (training_context['current_epoch']  % self.epoch_inteval == 0):
             self.plot_tile_image(training_context)
 
 
@@ -200,8 +202,7 @@ class SegTileImageCallback(VisualizationCallbackBase):
 
         # if self.tile_image_include_mask:
         #     tile_images_list.append(input*127.5+127.5)
-        tile_rgb_images(*tile_images_list, save_path=os.path.join(self.save_path, self.tile_image_name_prefix),
-                        imshow=True)
+        tile_rgb_images(*tile_images_list, save_path=os.path.join(self.save_path, self.tile_image_name_prefix),imshow=True)
 
     def on_batch_end(self, training_context):
         if self.batch_inteval > 0 and (training_context['current_batch']) % self.batch_inteval == 0:
@@ -268,33 +269,37 @@ class PrintGradientsCallback(VisualizationCallbackBase):
         self.lines = []
 
     def on_optimization_step_start(self, training_context):
-        if self.batch_inteval > 0 and (
-                training_context['current_epoch'] * training_context['total_batch'] + training_context[
-            'current_batch']) % self.batch_inteval == 0:
+        if self.batch_inteval > 0 and (training_context['current_epoch'] * training_context['total_batch'] + training_context['current_batch']) % self.batch_inteval == 0:
             grad_dict = {}
+            if 'grads_state' not in training_context:
+                training_context['grads_state']=OrderedDict()
+                training_context['grads_state']['first_layer']=[]
+                training_context['grads_state']['last_layer'] = []
             if self.first_layer != '' and self.last_layer != '':
                 for i, (k, v) in enumerate(training_context['current_model'].named_parameters()):
-                    if k == self.first_layer:
-                        training_context['grads_state']['first_layer'].append(
-                            np.abs(to_numpy(0 if v.grad is None else v.grad)).mean())
-                    elif k == self.last_layer:
-                        training_context['grads_state']['last_layer'].append(
-                            np.abs(to_numpy(0 if v.grad is None else v.grad)).mean())
+                    if v is not None and v.requires_grad == True:
+                        if k == self.first_layer:
+                            training_context['grads_state']['first_layer'].append(
+                                np.abs(to_numpy(0 if v.grad is None else v.grad)).mean())
+                        elif k == self.last_layer:
+                            training_context['grads_state']['last_layer'].append(
+                                np.abs(to_numpy(0 if v.grad is None else v.grad)).mean())
             else:
                 for i, (k, v) in enumerate(training_context['current_model'].named_parameters()):
-                    grad_dict[k] = np.abs(to_numpy(0 if v.grad is None else v.grad))
-                    if grad_dict[k].ndim > 1:
-                        if self.first_layer == '':
-                            self.first_layer = k
-                        else:
+                    if v is not None and v.requires_grad==True:
+                        grad_dict[k] = np.abs(to_numpy(0 if v.grad is None else v.grad))
+                        if grad_dict[k].ndim > 1:
+                            if self.first_layer == '':
+                                self.first_layer = k
                             self.last_layer = k
-
-                training_context['grads_state']['first_layer'].append(grad_dict[self.first_layer].mean())
-                training_context['grads_state']['last_layer'].append(grad_dict[self.last_layer].mean())
-
-            self.lines.append('{0:<16s}  first_layer gradients: {1:<8.3e}| last_layer gradients: {2:<8.3e}'.format(
-                training_context['current_model'].name, training_context['grads_state']['first_layer'][-1],
-                training_context['grads_state']['last_layer'][-1]))
+                if self.first_layer!='' and self.first_layer in grad_dict:
+                    training_context['grads_state']['first_layer'].append(grad_dict[self.first_layer].mean())
+                if self.last_layer != '' and self.last_layer in grad_dict:
+                    training_context['grads_state']['last_layer'].append(grad_dict[self.last_layer].mean())
+            if len(training_context['grads_state']['first_layer'])>0 and len(training_context['grads_state']['last_layer'])>0:
+                self.lines.append('{0:<16s}  first_layer gradients: {1:<8.3e}| last_layer gradients: {2:<8.3e}'.format(
+                    training_context['current_model'].name, training_context['grads_state']['first_layer'][-1],
+                    training_context['grads_state']['last_layer'][-1]))
 
     def on_overall_batch_end(self, training_context):
         if len(self.lines) > 0:
