@@ -13,14 +13,6 @@ if _image_backend=='opencv':
 else:
     from ..backend.pillow_backend import *
 
-from ..backend.load_backend import get_backend
-
-if get_backend()=='pytorch':
-    from ..backend.pytorch_backend import to_numpy,to_tensor
-elif get_backend()=='tensorflow':
-    from ..backend.tensorflow_backend import  to_numpy,to_tensor
-elif get_backend()=='cntk':
-    from ..backend.cntk_backend import  to_numpy,to_tensor
 
 __all__ = ['nms','xywh_to_xyxy','xyxy_to_xywh']
 
