@@ -1,7 +1,8 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-
+import numpy as np
+import pickle
 from .utils import *
 from ..backend.common import floatx
 from .data_provider import *
@@ -15,7 +16,7 @@ except ImportError:
     from six.moves.urllib.request import urlretrieve
 
 _session = get_session()
-_trident_dir = os.path.join(get_trident_dir(), 'datasets')
+_trident_dir = os.path.join(_session.trident_dir, 'datasets')
 _backend = _session.backend
 
 if 'TRIDENT_BACKEND' in os.environ:
@@ -131,36 +132,39 @@ def load_mnist(dataset_name='mnist', **kwargs):
         #     labels_file='emnist-balanced-'+labels_file
         #     images_file = 'emnist-balanced-' + images_file
 
-        download_file(base + labels_file, dirname, labels_file, dataset_name + '_labels_{0}'.format(kind))
-        download_file(base + images_file, dirname, images_file, dataset_name + '_images_{0}'.format(kind))
-        labels_path = os.path.join(dirname, labels_file)
-        images_path = os.path.join(dirname, images_file)
-        labeldata = None
-        imagedata = None
-        with gzip.open(labels_path, 'rb') as lbpath:
-            labels = np.frombuffer(lbpath.read(), dtype=np.uint8, offset=8)
-            labels = np.squeeze(labels).astype(np.int64)
-            labeldata = LabelDataset(labels.tolist())
+        is_data_download = download_file(base + labels_file, dirname, labels_file,
+                                         dataset_name + '_labels_{0}'.format(kind))
+        is_label_download = download_file(base + images_file, dirname, images_file,
+                                          dataset_name + '_images_{0}'.format(kind))
+        if is_data_download and is_label_download:
+            labels_path = os.path.join(dirname, labels_file)
+            images_path = os.path.join(dirname, images_file)
+            labeldata = None
+            imagedata = None
+            with gzip.open(labels_path, 'rb') as lbpath:
+                labels = np.frombuffer(lbpath.read(), dtype=np.uint8, offset=8)
+                labels = np.squeeze(labels).astype(np.int64)
+                labeldata = LabelDataset(labels.tolist())
 
-        with gzip.open(images_path, 'rb') as imgpath:
-            images = np.frombuffer(imgpath.read(), dtype=np.uint8, offset=16)
-            images = np.reshape(images, (len(labels), 784)).astype(dtype=_session.floatx)
-            images = np.reshape(images, (-1, 28, 28))
-            imagedata = ImageDataset(images, expect_data_type=ExpectDataType.gray,
-                                     get_image_mode=GetImageMode.processed)
-        if kind == 'train':
-            trainData = Iterator(data=imagedata, label=labeldata)
-        else:
-            testData = Iterator(data=imagedata, label=labeldata)
+            with gzip.open(images_path, 'rb') as imgpath:
+                images = np.frombuffer(imgpath.read(), dtype=np.uint8, offset=16)
+                images = np.reshape(images, (len(labels), 784)).astype(dtype=_session.floatx)
+                images = np.reshape(images, (-1, 28, 28))
+                imagedata = ImageDataset(images, expect_data_type=ExpectDataType.gray)
+            if kind == 'train':
+                trainData = Iterator(data=imagedata, label=labeldata)
+            else:
+                testData = Iterator(data=imagedata, label=labeldata)
 
-    dataset = DataProviderV2(dataset_name, traindata=trainData, testdata=testData)
+            dataset = DataProviderV2(dataset_name, traindata=trainData, testdata=testData)
+            dataset.binding_class_names(
+                [0, 1, 2, 3, 4, 5, 6, 7, 8, 9] if dataset_name == 'mnist' else ['T-shirt/top', 'Trouser', 'Pullover',
+                                                                                'Dress', 'Coat', 'Sandal', 'Shirt',
+                                                                                'Sneaker', 'Bag', 'Ankle boot'],
+                'en-US')
 
-    dataset.binding_class_names(
-        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9] if dataset_name == 'mnist' else ['T-shirt/top', 'Trouser', 'Pullover', 'Dress',
-                                                                        'Coat', 'Sandal', 'Shirt', 'Sneaker', 'Bag',
-                                                                        'Ankle boot'], 'en-US')
-
-    return dataset
+            return dataset
+        return None
 
 
 def load_cifar(dataset_name='cifar10'):
@@ -285,23 +289,22 @@ def load_text(filname, delimiter=',', skiprows=0, label_index=None, is_onehot=No
     return dataset
 
 
-def load_folder_images(dataset_name='', base_folder=None, shuffle=True, folder_as_label=True,
+def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=True, folder_as_label=True,
                        expect_data_type=ExpectDataType.rgb):
+    base_folder = sanitize_path(base_folder)
     if base_folder is not None and os.path.exists(base_folder):
-        base_folder = os.path.normpath(base_folder)
         print(base_folder)
         if folder_as_label == True:
-            sub_folders = glob.glob(base_folder + '/*/')
-
             class_names = []
-            if len(sub_folders) == 0:
-                raise ValueError('No subfolder in base folder.')
+            if classes is not None and isinstance(classes, list) and len(classes) > 0:
+                class_names.extend(classes)
             else:
-                if is_in_colab():
-                    class_names = [os.path.normpath(folder).split('/')[-1] for folder in sub_folders]
-                else:
-                    class_names = [os.path.normpath(folder).split('\\')[-1] for folder in sub_folders]
-                class_names = list(sorted(set(class_names)))
+                for subdir in sorted(os.listdir(base_folder)):
+                    if os.path.isdir(os.path.join(base_folder, subdir)):
+                        class_names.append(subdir)
+            if len(class_names) == 0:
+                raise ValueError('No subfolder in base folder.')
+            class_names = list(sorted(set(class_names)))
             print(class_names)
             labels = []
             imgs = []
@@ -310,11 +313,11 @@ def load_folder_images(dataset_name='', base_folder=None, shuffle=True, folder_a
                 class_imgs = glob.glob(base_folder + '/{0}/*.*g'.format(class_name))
                 print(base_folder + '/{0}/*.*g'.format(class_name))
                 print(len(class_imgs))
-                labels.extend((np.ones(len(class_imgs)) * i).astype(np.int64).tolist())
+                labels.extend([i] * len(class_imgs))
                 imgs.extend(class_imgs)
 
             imagedata = ImageDataset(imgs, expect_data_type=expect_data_type, get_image_mode=GetImageMode.processed)
-            print(len(imagedata))
+            print('extract {0} images...'.format(len(imagedata)))
             labelsdata = LabelDataset(labels)
             labelsdata.binding_class_names(class_names)
 
@@ -354,7 +357,7 @@ def load_stanford_cars(dataset_name='cars', kind='train', is_flatten=None, is_on
             pass
 
     download_file(train_url, dirname, train_url.split('/')[-1], dataset_name + '_images_{0}'.format('train'))
-    train_imgs_path = os.path.join(dirname, train_url.split('/')[-1])
+    tar_file_path = os.path.join(dirname, train_url.split('/')[-1])
 
     download_file(test_url, dirname, test_url.split('/')[-1], dataset_name + '_images_{0}'.format('test'))
     test_imgs_path = os.path.join(dirname, test_url.split('/')[-1])
@@ -405,12 +408,12 @@ def load_lfw(kind='train', is_flatten=None, is_onehot=None):
     download_file('http://vis-www.cs.umass.edu/lfw/pairsDevTrain.txt', dirname, 'pairsDevTrain.txt')
     download_file('http://vis-www.cs.umass.edu/lfw/pairsDevTest.txt', dirname, 'pairsDevTest.txt')
 
-    train_imgs_path = os.path.join(dirname, 'lfw.tgz')
+    tar_file_path = os.path.join(dirname, 'lfw.tgz')
     extract_path = os.path.join(_trident_dir, 'lfw')
-    extract_archive(train_imgs_path, dirname, archive_format='tar')
+    extract_archive(tar_file_path, dirname, archive_format='tar')
     dataset = load_folder_images(dataset_name, dirname)
 
-    extract_archive(train_imgs_path, dirname, archive_format='tar')
+    extract_archive(tar_file_path, dirname, archive_format='tar')
     dataset = load_folder_images(dataset_name, dirname)
     return dataset
 
@@ -418,7 +421,7 @@ def load_lfw(kind='train', is_flatten=None, is_onehot=None):
 def load_examples_data(dataset_name):
     dataset_name = dataset_name.strip().lower()
     if dataset_name.lower() not in ['pokemon', 'hanzi', 'animals', 'nsfw', 'simpsons', 'horse2zebra', 'people',
-                                    'autodrive','superresolution']:
+                                    'autodrive', 'superresolution', 'anpr', 'beauty']:
         raise ValueError('Not a  valid  dataset_name.')
     dataset_name = 'examples_' + dataset_name
     dirname = os.path.join(_trident_dir, dataset_name)
@@ -429,20 +432,22 @@ def load_examples_data(dataset_name):
             # Except permission denied and potential race conditions
             # in multi-threaded environments.
             pass
-
+    is_internet_ok = is_connected()
     if dataset_name == 'examples_pokemon':
-        download_file_from_google_drive('1U-xc54fX9j9BcidvRa0ow6qjssMlSF2A', dirname, 'pokemon.tar')
-        train_imgs_path = os.path.join(dirname, 'pokemon.tar')
+        is_download=download_file_from_google_drive('1U-xc54fX9j9BcidvRa0ow6qjssMlSF2A', dirname, 'pokemon.tar')
+        tar_file_path = os.path.join(dirname, 'pokemon.tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
         extract_path = os.path.join(dirname, 'pokemon')
-        extract_archive(train_imgs_path, dirname, archive_format='tar')
         dataset = load_folder_images(dataset_name, extract_path, folder_as_label=False)
         print('get pokemon images :{0}'.format(len(dataset)))
         return dataset
+
+
     elif dataset_name == 'examples_hanzi':
         download_file_from_google_drive('13UEzSG0az113gpRPKPyKrIE2HDaA2P4H', dirname, 'hanzi.tar')
-        train_imgs_path = os.path.join(dirname, 'hanzi.tar')
+        tar_file_path = os.path.join(dirname, 'hanzi.tar')
         extract_path = os.path.join(dirname, 'hanzi')
-        extract_archive(train_imgs_path, dirname, archive_format='tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
         dataset = load_folder_images(dataset_name, os.path.join(dirname, 'train'), folder_as_label=True,
                                      expect_data_type=ExpectDataType.gray)
 
@@ -455,15 +460,15 @@ def load_examples_data(dataset_name):
 
     elif dataset_name == 'examples_animals':
         download_file_from_google_drive('19Cjq8OO6qd9k9TMZxlPjDpejDOdiHJoW', dirname, 'animals.tar')
-        train_imgs_path = os.path.join(dirname, 'animals.tar')
-        extract_archive(train_imgs_path, dirname, archive_format='tar')
+        tar_file_path = os.path.join(dirname, 'animals.tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
         dataset = load_folder_images(dataset_name, dirname, folder_as_label=True)
         return dataset
     elif dataset_name == 'examples_nsfw':
         download_file_from_google_drive('1EXpV2QUrSFJ7zJn8NqtqFl1k6HvXsUzp', dirname, 'nsfw.tar')
-        train_imgs_path = os.path.join(dirname, 'nsfw.tar')
+        tar_file_path = os.path.join(dirname, 'nsfw.tar')
         extract_path = os.path.join(dirname, 'nsfw')
-        extract_archive(train_imgs_path, dirname, archive_format='tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
         trainData = np.load(os.path.join(dirname, 'train_porn_detector64_small.npy'), allow_pickle=True)
         testData = np.load(os.path.join(dirname, 'test_porn_detector64_small.npy'), allow_pickle=True)
 
@@ -486,18 +491,18 @@ def load_examples_data(dataset_name):
         return dataset
     elif dataset_name == 'examples_simpsons':
         download_file_from_google_drive('1hGNFbfBv3EZ4nx4Qod6PtSYzO8H4QIxC', dirname, 'simpsons.tar')
-        train_imgs_path = os.path.join(dirname, 'simpsons.tar')
+        tar_file_path = os.path.join(dirname, 'simpsons.tar')
         extract_path = os.path.join(dirname, 'simpsons')
-        extract_archive(train_imgs_path, extract_path, archive_format='tar')
+        extract_archive(tar_file_path, extract_path, archive_format='tar')
         dataset = load_folder_images(dataset_name, extract_path, folder_as_label=False)
         dataset.traindata.label = RandomNoiseDataset(shape=(100), random_mode='normal')
         print('get simpsons images :{0}'.format(len(dataset)))
         return dataset
     elif dataset_name == 'examples_horse2zebra':
         download_file_from_google_drive('1pqj-T90Vh4wVNBV09kYZWgVPsZUA2f7U', dirname, 'horse2zebra.tar')
-        train_imgs_path = os.path.join(dirname, 'horse2zebra.tar')
+        tar_file_path = os.path.join(dirname, 'horse2zebra.tar')
         extract_path = os.path.join(dirname, 'horse2zebra')
-        extract_archive(train_imgs_path, dirname, archive_format='tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
         trainA = ImageDataset(list_pictures(os.path.join(dirname, 'trainA')), expect_data_type=ExpectDataType.rgb,
                               get_image_mode=GetImageMode.processed)
         trainB = ImageDataset(list_pictures(os.path.join(dirname, 'trainB')), expect_data_type=ExpectDataType.rgb,
@@ -513,8 +518,8 @@ def load_examples_data(dataset_name):
         return dataset
     elif dataset_name == 'examples_people':
         download_file_from_google_drive('1H7mJJfWpmXpRxurMZQqY4N_UXWLbQ2pT', dirname, 'people.tar')
-        train_imgs_path = os.path.join(dirname, 'people.tar')
-        extract_archive(train_imgs_path, dirname, archive_format='tar')
+        tar_file_path = os.path.join(dirname, 'people.tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
         imgs = glob.glob(os.path.join(dirname, 'imgs', '*.*g'))
         masks = glob.glob(os.path.join(dirname, 'masks', '*.png'))
         # make_dir_if_need(os.path.join(dirname, 'trimap'))
@@ -531,41 +536,108 @@ def load_examples_data(dataset_name):
         return dataset
     elif dataset_name == 'examples_autodrive':
         download_file_from_google_drive('1JqPPeHqhWLqnI6bD8nuHcVx-Y56oIZMK', dirname, 'autodrive.tar')
-        train_imgs_path = os.path.join(dirname, 'autodrive.tar')
+        tar_file_path = os.path.join(dirname, 'autodrive.tar')
         extract_path = os.path.join(dirname, 'autodrive')
-        extract_archive(train_imgs_path, dirname, archive_format='tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
         imgs = glob.glob(os.path.join(dirname, 'images', '*.*g'))
         masks = glob.glob(os.path.join(dirname, 'masks', '*.png'))
 
-
-
         imgdata = ImageDataset(images=imgs, expect_data_type=ExpectDataType.rgb)
         mskdata = MaskDataset(masks=masks, expect_data_type=ExpectDataType.color_mask)
+
         def parse_code(l):
             if len(l.strip().split("\t")) == 2:
                 a, b = l.replace('\t\t', '\t').strip().split("\t")
                 return tuple(int(i) for i in b.split(' ')), a
 
-        label_codes, label_names = zip(*[parse_code(l) for l in open(os.path.join(dirname,"label_colors.txt")).readlines()])
+        label_codes, label_names = zip(
+            *[parse_code(l) for l in open(os.path.join(dirname, "label_colors.txt")).readlines()])
         for i in range(len(label_codes)):
-            mskdata.palette[label_names[i]]=label_codes[i]
+            mskdata.palette[label_names[i]] = label_codes[i]
 
         dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=mskdata))
         print('get autodrive images :{0}'.format(len(dataset)))
         return dataset
     elif dataset_name == 'examples_superresolution':
         download_file_from_google_drive('1v1uoymrWI_MLSiGvSGW7tWJYSnzzXpEQ', dirname, 'superresolution.tar')
-        train_imgs_path = os.path.join(dirname, 'superresolution.tar')
-        extract_archive(train_imgs_path, dirname, archive_format='tar')
-        imgs = glob.glob(os.path.join(dirname,  '*.*g'))
+        tar_file_path = os.path.join(dirname, 'superresolution.tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
+        imgs = glob.glob(os.path.join(dirname, '*.*g'))
         imgs.extend(glob.glob(os.path.join(dirname, '*.bmp')))
         print('get super resolution images :{0}'.format(len(imgs)))
-        imgdata = ImageDataset(images=imgs*2, expect_data_type=ExpectDataType.rgb,symbol='lr')
-        labeldata = ImageDataset(images=imgs*2, expect_data_type=ExpectDataType.rgb,symbol='hr')
-        dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata,label=labeldata))
-
-
+        imgdata = ImageDataset(images=imgs * 2, expect_data_type=ExpectDataType.rgb, symbol='lr')
+        labeldata = ImageDataset(images=imgs * 2, expect_data_type=ExpectDataType.rgb, symbol='hr')
+        dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=labeldata))
         return dataset
+    elif dataset_name == 'examples_beauty':
+        download_file_from_google_drive('1aJhxN9IqsxuayhRTm-gmxk6PiLe5wm9X', dirname, 'beauty.tar')
+        tar_file_path = os.path.join(dirname, 'beauty.tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
+        # 讀取圖片數據
+        images_dict = {}
+        with open(os.path.join(dirname, 'images_dict.pkl'), 'rb') as fp:
+            images_dict = pickle.load(fp)
+
+        f = open(os.path.join(dirname, 'All_Ratings.txt'), encoding='utf-8-sig').readlines()
+        imgs = []
+        landmark = []
+        ratings = []
+        for row in f:
+            data = row.strip().split('\t')
+            if 'images\\' + data[0] in images_dict:
+                img = images_dict['images\\' + data[0]][0]
+                img = img.transpose([2, 0, 1])[::-1].transpose([1, 2, 0])
+                imgs.append(img)
+                landmark = images_dict['images\\' + data[0]][1].astype(np.float32) / 256.0
+                rating = np.array([(float(data[1])) / 5.00])
+                rating = np.concatenate([landmark.reshape(-1), rating], axis=0).astype(np.float32)
+                ratings.append(rating)
+        print('{0} faces loaded...'.format(len(imgs)))
+        imgdata = ImageDataset(images=imgs, expect_data_type=ExpectDataType.rgb, symbol='faces')
+        labeldata = NumpyDataset(data=ratings, expect_data_type=ExpectDataType.array_data, symbol='ratings')
+        data_provider = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=labeldata))
+        return data_provider
+
+    elif dataset_name == 'examples_anpr':
+        download_file_from_google_drive('1uGBd8tXlP0TZAXNgrR6H0jl5MXj7VPbN', dirname, 'anpr.tar')
+        tar_file_path = os.path.join(dirname, 'anpr.tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
+        imgs = glob.glob(os.path.join(dirname, '*.*g'))
+        # CCPD (Chinese City Parking Dataset, ECCV) and PDRC (license Plate Detection and Recognition Challenge)
+        # https://github.com/detectRecog/CCPD
+        provinces = ["皖", "沪", "津", "渝", "冀", "晋", "蒙", "辽", "吉", "黑", "苏", "浙", "京", "闽", "赣", "鲁", "豫", "鄂", "湘", "粤",
+                     "桂", "琼", "川", "贵", "云", "藏", "陕", "甘", "青", "宁", "新", "警", "学", "O"]
+        alphabets = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V',
+                     'W', 'X', 'Y', 'Z', 'O']
+        ads = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',
+               'X', 'Y', 'Z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'O']
+
+        def lp2char(lp):
+            cols = lp.split('_')
+            charstring = ''
+            for i in range(len(cols)):
+                if i == 0:
+                    charstring += provinces[int(cols[i])]
+                elif i == 1:
+                    charstring += alphabets[int(cols[i])]
+                else:
+                    charstring += ads[int(cols[i])]
+            return charstring
+
+        width = 720
+        height = 1160
+        for im_path in imgs:
+            lbl = im_path.split('/')[-1].rsplit('.', 1)[0].split('-')[-3]
+            charstring = lp2char(lbl)
+            iname = im_path.rsplit('/', 1)[-1].rsplit('.', 1)[0].split('-')
+            [leftUp, rightDown] = [[int(eel) for eel in el.split('&')] for el in iname[2].split('_')]
+            box = [leftUp[0], leftUp[1], rightDown[0], rightDown[1]]
+            ori_w, ori_h = [float(int(el)) for el in [width, height]]
+            new_labels = [(leftUp[0] + rightDown[0]) / (2 * ori_w), (leftUp[1] + rightDown[1]) / (2 * ori_h),
+                          (rightDown[0] - leftUp[0]) / ori_w, (rightDown[1] - leftUp[1]) / ori_h]
+
+
+
 
     else:
         return None
\ No newline at end of file
