@@ -340,8 +340,9 @@ class DataProviderV2(object):
         self._label_transform_funcs = []
         self._paired_transform_funcs = []
         self.spatial_transform_funcs = []
-        if not isinstance(self.traindata.data,ImageDataset):
-            del self["my_property"]
+
+
+
 
     @property
     def signature(self):
@@ -525,7 +526,12 @@ class DataProviderV2(object):
             return 0
 
     def next(self):
-        return self.__next__()
+        if self.scenario == 'test' and self.testdata is not None:
+            result = self.testdata.next()
+            return result
+        else:
+            result= self.traindata.next()
+            return result
 
     def __next__(self):
         if self.scenario == 'test' and self.testdata is not None:
@@ -534,11 +540,11 @@ class DataProviderV2(object):
             return next(self.traindata)
 
     def next_train(self):
-        return next(self.traindata)
+        return self.traindata.next()
 
     def next_test(self):
         if self.testdata is not None:
-            return next(self.testdata)
+            return self.testdata.next()
         else:
             return None
 
