@@ -274,9 +274,10 @@ class LabelDataset(Dataset):
             if language is None:
                 language = 'en-us'
             self.class_names[language] = list(class_names)
+
             self.__default_language__ = language
-            self._lab2idx = {v: k for k, v in enumerate(self.class_names[language])}
-            self._idx2lab = {k: v for k, v in enumerate(self.class_names[language])}
+            self._lab2idx = dict(zip(self.class_names[language], range(len(self.class_names[language]))))
+            self._idx2lab = dict(zip(range(len(self.class_names[language])),self.class_names[language]))
 
     def __getitem__(self, index: int):
         label = super().__getitem__(index)
@@ -389,7 +390,7 @@ class SsdBboxDataset(BboxDataset):
         return np.concatenate([(center_form_boxes[..., :2] - center_form_priors[..., :2]) / center_form_priors[...,
                                                                                             2:] / self.center_variance,
                                np.log(center_form_boxes[..., 2:] / center_form_priors[..., 2:]) / self.size_variance],
-            axis=len(center_form_boxes.shape) - 1)
+                              axis=len(center_form_boxes.shape) - 1)
 
     def assign_priors(self, gt_boxes, gt_labels, center_form_priors, iou_threshold):
         # """Assign ground truth boxes and targets to priors.
@@ -514,7 +515,7 @@ class Iterator(object):
             self._label = label
             if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data,
                                                                                                 ImageDataset) and len(
-                    self._label) == len(self._data):
+                self._label) == len(self._data):
                 self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
             else:
                 self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
@@ -536,9 +537,7 @@ class Iterator(object):
     @data.setter
     def data(self, value):
         self._data = value
-        if self._label is not None and isinstance(self._label, (MaskDataset, ImageDataset)) and isinstance(self._data,
-                                                                                                           ImageDataset) and len(
-            self._label) == len(self._data):
+        if self._label is not None and isinstance(self._label, (MaskDataset,BboxDataset,ImageDataset)) and isinstance(self._data, ImageDataset) and len(self._label) == len(self._data):
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
         else:
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
@@ -553,14 +552,12 @@ class Iterator(object):
     @label.setter
     def label(self, value):
         self._label = value
-        if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data,
-                                                                                            ImageDataset) and len(
-                self._label) == len(self._data):
+        if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data, ImageDataset) and len(
+            self._label) == len(self._data):
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = True
         else:
             self._label.is_pair_process = self._data.is_pair_process = self.is_pair_process = False
 
-        self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
 
     @property
@@ -570,13 +567,11 @@ class Iterator(object):
     @unpair.setter
     def unpair(self, value):
         self._unpair = value
-        self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
 
     @property
     def palette(self):
-        if isinstance(self._label, MaskDataset) and self._label.expect_data_type in [ExpectDataType.label_mask,
-                                                                                     ExpectDataType.color_mask]:
+        if isinstance(self._label, MaskDataset) and self._label.expect_data_type in [ExpectDataType.label_mask,   ExpectDataType.color_mask]:
             return self._label.palette
         else:
             return None
@@ -640,7 +635,7 @@ class Iterator(object):
             return img_data, paired_img
 
     def __getitem__(self, index: int):
-        #start = time.time()
+        # start = time.time()
 
         try:
             bbox = None
@@ -684,14 +679,10 @@ class Iterator(object):
                 # print('paired_transform:{0}'.format(stop - start))
                 # start = stop
                 if hasattr(self.data, 'image_transform'):
-                    data = self.data.image_transform(data)
-                    # stop = time.time()
-                    # print('data image_transform:{0}'.format(stop - start))
+                    data = self.data.image_transform( data)  # stop = time.time()  # print('data image_transform:{0}'.format(stop - start))  #
                     # start = stop
                 if hasattr(self.label, 'image_transform'):
-                    label = self.label.image_transform(label)
-                    # stop = time.time()
-                    # print('label image_transform:{0}'.format(stop - start))
+                    label = self.label.image_transform(label)  # stop = time.time()  # print('label image_transform:{0}'.format(stop - start))  #
                     # start = stop
             else:
                 if hasattr(self.label, 'label_transform'):
@@ -715,17 +706,11 @@ class Iterator(object):
                     self.signature['mask' if self.label.symbol is None or len(
                         self.label.symbol) == 0 else self.label.symbol] = mask.shape
                 if label is not None:
-                    self.signature['label' if self.label.symbol is None or len(
-                        self.label.symbol) == 0 else self.label.symbol] = label.shape if isinstance(label,
-                                                                                                    np.ndarray) else \
-                        type(
-                        label)
+                    self.signature['label' if self.label.symbol is None or len(self.label.symbol) == 0 else self.label.symbol] = label.shape if isinstance(label, np.ndarray) else type(label)
                 if unpair is not None:
                     self.signature['unpair' if self.unpair.symbol is None or len(
-                        self.unpair.symbol) == 0 else self.unpair.symbol] = unpair.shape
-                # stop = time.time()
-                # print('signature:{0}'.format(stop - start))
-                # start = stop
+                        self.unpair.symbol) == 0 else self.unpair.symbol] = unpair.shape  # stop = time.time()  #
+                    # print('signature:{0}'.format(stop - start))  # start = stop
 
             if data is not None:
                 return_data.append(data)
@@ -744,45 +729,40 @@ class Iterator(object):
         except:
             PrintException()
 
-
     def _next_index(self):
         return next(self._sample_iter)
 
     def __iter__(self):
         return self._sample_iter
 
+    # return a batch , do minimal fetch before return
     def next(self):
         if self.out_queue.qsize() == 0:
             in_data = self._sample_iter.__next__()
-            self.out_queue.put(in_data,False)
-            #self.out_queue.task_done()
+            self.out_queue.put(in_data, False)
 
         out_data = self.out_queue.get(False)
-        #self.out_queue.task_done()
 
         if self.out_queue.qsize() <= self.buffer_size // 2:
             for i in range(2):
                 in_data = self._sample_iter.__next__()
-                self.out_queue.put(in_data,False)
-                #self.out_queue.task_done()
+                self.out_queue.put(in_data, False)
+
         return out_data
 
+    # yield a batch , and trigger following fetch after yield
     def __next__(self):
         if self.out_queue.qsize() == 0:
             in_data = self._sample_iter.__next__()
-            self.out_queue.put(in_data,False)
-            #self.out_queue.task_done()
+            self.out_queue.put(in_data, False)
 
         out_data = self.out_queue.get(False)
-        #self.out_queue.task_done()
+
         yield out_data
         if self.out_queue.qsize() <= self.buffer_size // 2:
-            for i in range(self.buffer_size -self.out_queue.qsize()):
+            for i in range(self.buffer_size - self.out_queue.qsize()):
                 in_data = self._sample_iter.__next__()
-                self.out_queue.put(in_data,False)
-                #self.out_queue.task_done()
-
-
+                self.out_queue.put(in_data, False)
 
     def __len__(self):
         return max([len(self.data) if self.data is not None else 0, len(self.unpair) if self.unpair is not None else 0])
