@@ -19,8 +19,8 @@ from itertools import repeat
 import cv2
 
 __all__ = ['read_image','read_mask','save_image','save_mask','image2array','array2image','mask2array','array2mask','list_pictures','normalize','unnormalize','channel_reverse','blur','random_blur','random_crop','resize','rescale','downsample_then_upsample','add_noise','gray_scale','to_rgb','auto_level','random_invert_color',
-           'image_backend_adaptive','reverse_image_backend_adaptive','random_adjust_hue', 'random_channel_shift', 'random_cutout','random_rescale_crop','random_center_crop','adjust_gamma','random_adjust_gamma','adjust_contast','random_adjust_contast','clahe','erosion_then_dilation','dilation_then_erosion','image_erosion','image_dilation','adaptive_binarization','random_transform',
-           'horizontal_flip','random_mirror']
+           'image_backend_adaptive','reverse_image_backend_adaptive','random_adjust_hue', 'random_channel_shift', 'random_cutout','random_rescale_crop','random_center_crop','adjust_gamma','random_adjust_gamma','adjust_contrast','random_adjust_contrast','clahe','erosion_then_dilation','dilation_then_erosion','image_erosion','image_dilation','adaptive_binarization','random_transform',
+           'horizontal_flip','random_mirror','to_low_resolution']
 
 
 _session=get_session()
@@ -32,14 +32,7 @@ if _image_backend=='opencv':
 else:
     from ..backend.pillow_backend import *
 
-from ..backend.load_backend import get_backend
 
-if get_backend()=='pytorch':
-    from ..backend.pytorch_backend import to_numpy,to_tensor
-elif get_backend()=='tensorflow':
-    from ..backend.tensorflow_backend import  to_numpy,to_tensor
-elif get_backend()=='cntk':
-    from ..backend.cntk_backend import  to_numpy,to_tensor
 
 
 
@@ -86,8 +79,6 @@ def random_augmentation(func):
 
 
 
-
-
 def add_noise(intensity=0.1):
     def img_op(image:np.ndarray):
         rr=random.randint(0,10)
@@ -321,7 +312,7 @@ def random_adjust_gamma(gamma=(0.6,1.4)):
     return img_op
 
 
-def adjust_contast(alpha=1):
+def adjust_contrast(alpha=1):
     beta=0
     def img_op(image: np.ndarray):
         image = image.astype(np.float32) * alpha + beta
@@ -330,7 +321,7 @@ def adjust_contast(alpha=1):
     return img_op
 
 
-def random_adjust_contast(scale=(0.5, 1.5)):
+def random_adjust_contrast(scale=(0.5, 1.5)):
     beta=0
     scalemin,scalemax=scale
     def img_op(image: np.ndarray):
@@ -806,6 +797,27 @@ def random_transform(rotation_range= 12, zoom_range= 0.05, shift_range=0.05,rand
         return img
     return img_op
 
+
+def to_low_resolution(scale=2):
+    def img_op(image: np.ndarray):
+        rnd=random.randint(0,10)
+        if rnd%3==0:
+            image=downsample_then_upsample(scale+2)(image)
+            image=add_noise(0.02)(image)
+            image=rescale(1/scale)(image)
+        elif rnd % 3 == 1:
+            image = add_noise(0.02)(image)
+            image = random_blur((0.1,0.5))(image)
+            image = rescale(1 / scale)(image)
+        elif rnd % 3 == 2:
+            image = downsample_then_upsample(scale,repeat=2)(image)
+            image = add_noise(0.02)(image)
+            image = rescale(1 / scale)(image)
+        return image
+    return img_op
+
+
+
 # def image_smoothening():
 #     def img_op(image: np.ndarray):
 #         ret1, th1 = cv2.threshold(img, BINARY_THREHOLD, 255, cv2.THRESH_BINARY)
