@@ -4,8 +4,15 @@ except ImportError:
     import queue as Queue
 import threading
 import time
-import matplotlib
-matplotlib.use('Agg')
+from ..misc.ipython_utils import is_in_ipython,is_in_colab
+if is_in_ipython():
+    from IPython import display
+
+if not is_in_colab:
+    import matplotlib
+    matplotlib.use('Qt5Agg' if not is_in_ipython() else 'TkAgg')
+else:
+    import matplotlib
 import matplotlib.pyplot as plt
 import itertools
 from .image_common import list_pictures
