@@ -9,14 +9,13 @@ import numpy as np
 import math
 
 from ..backend.common import *
-
+from ..backend.load_backend import *
 
 __all__ = ['label_backend_adaptive','get_onehot','check_is_onehot']
 
 
 _session=get_session()
-_backend=_session.backend
-_image_backend=_session.image_backend
+
 
 
 def get_onehot(idx,len):
@@ -37,8 +36,7 @@ def check_is_onehot(label):
         return False
 
 def label_backend_adaptive(label,label_mapping=None,expect_image_type=None):
-    if _backend == 'pytorch':
-
+    if get_backend() == 'pytorch':
         if isinstance(label,np.ndarray):
             # binary mask
             if expect_image_type == ExpectDataType.binary_mask:
@@ -59,17 +57,18 @@ def label_backend_adaptive(label,label_mapping=None,expect_image_type=None):
                     if check_is_onehot(label):
                         label=np.argmax(label,-1).astype(np.int64)
                         return label
-
-
             label = label.astype(np.int64)
         elif isinstance(label, int):
             return label
         return label
-    elif label_mapping is None:
-        return label
-    else:
-        if isinstance(label_mapping,dict) and len(label_mapping)>0:
-            label_mapping=list(label_mapping.values())[0]
-        label=get_onehot(label,len(label_mapping))
+    elif get_backend()== 'tensorflow':
+        if isinstance(label, int):
+            if isinstance(label_mapping, dict) and len(label_mapping) > 0:
+                label_mapping = list(label_mapping.values())[0]
+                label = get_onehot(label, len(label_mapping))
+                return label
+            elif label_mapping is None:
+                return label
     return label
 
+
