@@ -12,13 +12,6 @@ from ..backend.common import OrderedDict
 from .image_common import check_same_size
 from ..backend.load_backend import get_backend
 
-if get_backend()=='pytorch':
-    from ..backend.pytorch_backend import to_numpy,to_tensor
-elif get_backend()=='tensorflow':
-    from ..backend.tensorflow_backend import  to_numpy,to_tensor
-elif get_backend()=='cntk':
-    from ..backend.cntk_backend import  to_numpy,to_tensor
-
 
 
 __all__ = ['Sampler','SequentialSampler','RandomSampler','BatchSampler']
@@ -114,7 +107,8 @@ class BatchSampler(Sampler):
         [[0, 1, 2], [3, 4, 5], [6, 7, 8]]
     """
 
-    def __init__(self, data_source, batch_size=1, is_shuffle=True,drop_last=False):
+    def __init__(self, data_source, batch_size=1, is_shuffle=True, drop_last=False):
+        super().__init__(data_source)
         if not isinstance(batch_size, int) or isinstance(batch_size, bool) or batch_size <= 0:
             raise ValueError("batch_size should be a positive integeral value, "
                              "but got batch_size={}".format(batch_size))
@@ -128,9 +122,9 @@ class BatchSampler(Sampler):
 
 
         idxes = np.arange(len(self.data_source))
-        if len(self.data_source) % self.batch_size>0:
-            idxes=idxes[:-(len(self.data_source) % self.batch_size)]
-        if self.is_shuffle==True:
+        if len(self.data_source) % self.batch_size > 0:
+            idxes = idxes[:-(len(self.data_source) % self.batch_size)]
+        if self.is_shuffle == True:
             np.random.shuffle(idxes)
         idxes = list(idxes)
 
@@ -166,7 +160,8 @@ class BatchSampler(Sampler):
                 batch = OrderedDict()
                 batch_shape = OrderedDict()
                 _data_cnt = 0
-        if len(batch)==0:
+        if len(batch) == 0:
+            self.reset()
             raise StopIteration
 
     def __len__(self):
@@ -184,3 +179,4 @@ class BatchSampler(Sampler):
         idxes = list(idxes)
         self.sampler = iter(idxes)
 
+
