@@ -1,8 +1,3 @@
-'''
- utils for loading dataset
-'''
-
-
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
@@ -17,7 +12,6 @@ from  sys import stderr
 import requests
 import warnings
 import numpy as np
-from socket import create_connection
 from tqdm import tqdm
 import gzip
 import tarfile
@@ -33,6 +27,7 @@ from ..backend.common import OrderedDict,PrintException
 from six.moves.urllib.error import HTTPError
 from six.moves.urllib.error import URLError
 from six.moves.urllib.request import urlopen
+import urllib.request
 try:
     from urllib.request import urlretrieve
 except ImportError:
@@ -40,11 +35,12 @@ except ImportError:
 
 def is_connected():
     try:
-        urlopen('https://docs.google.com')
+        urllib.request.urlopen('https://google.com') #Python 3.x
         return True
     except :
         return False
 
+
 def ensure_dir(dirpath):
     if not os.path.exists(dirpath): os.makedirs(dirpath)
 
@@ -114,29 +110,24 @@ def download_file(src, dirname, filename, desc=''):
     _h = _read_h(dirname)
     if os.path.exists(os.path.join(dirname, filename)) and _h != {} and _h.get('is_downloaded', False) == True:
         print('archive file is already existing, donnot need download again.')
+        return True
     else:
         if os.path.exists(os.path.join(dirname, filename)):
             os.remove(os.path.join(dirname, filename))
-        error_msg = 'URL fetch failure on {}: {} -- {}'
-        try:
-            try:
-                with TqdmProgress(unit='B', unit_scale=True,  leave=True,miniters=10, desc=desc) as t:  # all optional kwargs
-                    urlretrieve(src, filename=os.path.join(dirname, filename), reporthook=t.update_to, data=None)
-                    _write_h(dirname, True, False)
-            except HTTPError as e:
-                _write_h(dirname, False, False)
-                raise Exception(error_msg.format(src, os.path.join(dirname, filename), e.code, e.__str__()))
 
-            except URLError as e:
-                _write_h(dirname, False, False)
-                raise Exception(error_msg.format(src, os.path.join(dirname, filename), e.errno, e.reason))
-            except (Exception, KeyboardInterrupt) as e:
-                _write_h(dirname, False, False)
-                if os.path.exists(os.path.join(dirname, filename)):
-                    os.remove(os.path.join(dirname, filename))
-        except:
+        try:
+            with TqdmProgress(unit='B', unit_scale=True,  leave=True,miniters=10, desc=desc) as t:  # all optional kwargs
+                urlretrieve(src, filename=os.path.join(dirname, filename), reporthook=t.update_to, data=None)
+                _write_h(dirname, True, False)
+            return True
+        except Exception as e:
             _write_h(dirname, False, False)
-            raise
+            print('***Cannot download data, so the data provider cannot initialized.\n',flush=True)
+            print('***Please check your internet or download  files from following url in another computer, \n and then put them into {0}\n {1} '.format( dirname,src),flush=True)
+
+            print(e)
+            return False
+
 
 
 
@@ -161,20 +152,25 @@ def download_file_from_google_drive(file_id, dirname, filename=None, md5=None):
     #     print('archive file is already existing, donnot need download again.')
     if os.path.exists(os.path.join(dirname, filename)) and _h!={} and  _h.get('is_downloaded', False)==True:
         print('archive file is already existing, donnot need download again.')
+        return True
     else:
-        with TqdmProgress(unit='B', unit_scale=True, miniters=10, desc='') as t:  # all optional kwargs
+        try:
             session = requests.Session()
             response = session.get(url, params={'id': file_id}, stream=True)
             token = _get_confirm_token(response)
-
             if token:
                 params = {'id': file_id, 'confirm': token}
                 response = session.get(url, params=params, stream=True)
-
             _save_response_content(response, fpath)
             _write_h(dirname,True,False)
             return True
-    return False
+        except Exception as e:
+            print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
+            print( '***Please check your internet or download  files from following url in another computer, \n and then put them into {0}\n {1} '.format(dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
+            Print(e)
+            return False
+
+
 
 
 def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
@@ -182,7 +178,7 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
 
     Args:
         file_id (str): id of file to be downloaded
-        root (str): Directory to place downloaded file in
+        dirname (str): Directory to place downloaded file in
         filename (str, optional): Name to save the file under. If None, use the id of the file.
         md5 (str, optional): MD5 checksum of the download. If None, do not check
     """
@@ -196,23 +192,34 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
     isload = False
     models_md5 = None
     need_download = True
-    if os.path.exists(os.path.join(dirname, 'models_md5.json')):
-        os.remove(os.path.join(dirname, 'models_md5.json'))
+
     try:
-        isload = download_file_from_google_drive('1AujFsfyzyYUcUrNI_GT1dvNrHaXIQkGz', dirname, 'models_md5.json')
+        session = requests.Session()
+        response = session.get(url, params={'id': '1AujFsfyzyYUcUrNI_GT1dvNrHaXIQkGz'}, stream=False)
+        if os.path.exists(os.path.join(dirname, 'models_md5.json')):
+            os.remove(os.path.join(dirname, 'models_md5.json'))
+        with open(os.path.join(dirname, 'models_md5.json'),"wb") as f:
+            f.write(response.content)
+
         check_internet = True
         if os.path.exists(os.path.join(dirname, 'models_md5.json')):
             with open(os.path.join(dirname, 'models_md5.json')) as f:
                 models_md5 = json.load(f)
     except Exception:
+        PrintException()
         check_internet=False
 
+
     if check_internet == False:
         if os.path.exists(os.path.join(dirname, filename)):
             print('internet connect  error,model file is already existing, donnot need download again.')
             return True
         else:
-            raise NewConnectionError
+            print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
+            print('***Please check your internet or download  files from following url in another computer, \n and then '
+                'put them into {0}\n {1} '.format( dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
+
+        return False
     else:
 
         try:
@@ -229,17 +236,25 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
                         need_download = False
 
             if need_download:
-                with TqdmProgress(unit='B', unit_scale=True, miniters=10, desc='') as t:  # all optional kwargs
-                    session = requests.Session()
-                    response = session.get(url, params={'id': file_id}, stream=True)
-                    token = _get_confirm_token(response)
-                    if token:
-                        params = {'id': file_id, 'confirm': token}
-                        response = session.get(url, params=params, stream=True)
-                    _save_response_content(response, fpath)
+                session = requests.Session()
+                response = session.get(url, params={'id': file_id}, stream=True)
+                token = _get_confirm_token(response)
+                if token:
+                    params = {'id': file_id, 'confirm': token}
+                    response = session.get(url, params=params, stream=True)
+                _save_response_content(response, fpath)
+
+                if check_integrity(os.path.join(dirname, filename), models_md5[filename]):
+                    print('model file is downloaded and validated.')
+                else:
+                    print('model file is downloaded but not match md5.')
         except Exception as e:
+            print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
+            print('***Please check your internet or download  files from following url in another computer, \n and then '
+                'put them into {0}\n {1} '.format(dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
             print(e)
-            PrintException()
+
+        return False
 
 
 def _get_confirm_token(response):
@@ -251,10 +266,11 @@ def _get_confirm_token(response):
 
 
 def _save_response_content(response, destination, chunk_size=32768):
+    folder, file=os.path.split(destination)
+    progress=0
     with open(destination, "wb") as f:
-        pbar = tqdm(total=None)
-        progress = 0
-        for chunk in response.iter_content(chunk_size):
+        pbar = TqdmProgress(response.iter_content(chunk_size=chunk_size), total=None, unit='MB',unit_scale=True, miniters=10,desc=file, leave=True, file=sys.stdout)
+        for chunk in pbar:
             if chunk:  # filter out keep-alive new chunks
                 f.write(chunk)
                 progress += len(chunk)
@@ -262,12 +278,14 @@ def _save_response_content(response, destination, chunk_size=32768):
         pbar.close()
 
 
-def extract_archive(file_path, path='.', archive_format='auto'):
+
+
+def extract_archive(file_path, target_folder=None, archive_format='auto'):
   """Extracts an archive if it matches tar, tar.gz, tar.bz, or zip formats.
 
   Arguments:
       file_path: path to the archive file
-      path: path to extract the archive file
+      target_folder: path to extract the archive file
       archive_format: Archive format to try for extracting the file.
           Options are 'auto', 'tar', 'zip', and None.
           'tar' includes tar, tar.gz, and tar.bz files.
@@ -278,7 +296,8 @@ def extract_archive(file_path, path='.', archive_format='auto'):
       True if a match was found and an archive extraction was completed,
       False otherwise.
   """
-  _h = _read_h(path)
+  folder,_=os.path.split(file_path)
+  _h = _read_h(target_folder)
   if _h != {} and _h.get('is_extracted', False) == True:
       print('extraction is finished, donnot need extract again.')
       return True
@@ -300,24 +319,21 @@ def extract_archive(file_path, path='.', archive_format='auto'):
       is_match_fn = zipfile.is_zipfile
 
     if is_match_fn(file_path):
-      with open_fn(file_path) as archive:
-        try:
-          if os.path.exists(os.path.join(path,file_path.split('\\')[-1].strip().split('.')[0])) and os.path.isdir(path):
-              print('extracted folder is already existing, donnot need extract again.')
-          else:
-            archive.extractall(path)
-            _write_h(path, True, True)
-        except (tarfile.TarError, RuntimeError, KeyboardInterrupt):
-            _write_h(path, False, False)
-            if os.path.exists(path):
-                if os.path.isfile(path):
-                    os.remove(path)
-                else:
-                    shutil.rmtree(path)
-            raise
+        print('Starting to decompress the archive....')
+        with open_fn(file_path) as archive:
+            try:
+                archive.extractall(target_folder)
+                _write_h(target_folder, True, True)
+            except (tarfile.TarError, RuntimeError, KeyboardInterrupt):
+                sys.stderr.write('Decompressing the archive is not success')
+                PrintException()
+                _write_h(target_folder, True, False)
+                if os.path.exists(target_folder):
+                    shutil.rmtree(target_folder)
+                raise
         return True
     else:
-        _write_h(path, False, False)
+        _write_h(target_folder, False, False)
   return False
 
 
