@@ -474,9 +474,10 @@ class Conv2d(_ConvNd):
 
         if self.padding_mode == 'circular':
             expanded_padding = ((self.padding[1] + 1) // 2, self.padding[1] // 2, (self.padding[0] + 1) // 2, self.padding[0] // 2)
-            x=F.pad(input, expanded_padding, mode='circular')
+            x=F.pad(x, expanded_padding, mode='circular')
         else:
             x = F.pad(x, (self.padding[1] , self.padding[1], self.padding[0] , self.padding[0] ), mode='constant' if self.padding_mode == 'zero' else self.padding_mode)
+
         return F.conv2d(x, self.weight, self.bias, self.strides, _pair(0), self.dilation, self.groups)
 
     def forward(self, *x):
@@ -526,13 +527,18 @@ class Conv3d(_ConvNd):
             pad_w = self.padding[2] * 2
 
         if self.padding_mode == 'circular':
-            self.padding = ((pad_z + 1) // 2, pad_z // 2, (pad_h + 1) // 2, pad_h // 2, (pad_w + 1) // 2, pad_w // 2)
+            self.padding = (pad_z // 2, pad_z-(pad_z // 2), pad_h // 2, pad_h-(pad_h // 2), pad_w// 2,pad_w- (pad_w // 2))
         else:
             self.padding = (pad_z // 2, pad_h // 2, pad_w // 2)
 
     def conv3d_forward(self, x):
-        x = F.pad(x, self.padding, mode='constant' if self.padding_mode=='zero' else self.padding_mode)
-        return F.conv3d(x, self.weight, self.bias, self.strides, self.padding, self.dilation, self.groups)
+        if self.padding_mode == 'circular':
+            expanded_padding = ((self.padding[2] + 1) // 2, self.padding[2] // 2, (self.padding[1] + 1) // 2, self.padding[1] // 2,(self.padding[0] + 1) // 2, self.padding[0] // 2)
+            x = F.pad(x, expanded_padding, mode='circular')
+        else:
+            x = F.pad(x, (self.padding[2], self.padding[2],self.padding[1], self.padding[1], self.padding[0], self.padding[0]),mode='constant' if self.padding_mode == 'zero' else self.padding_mode)
+
+        return F.conv3d(x, self.weight, self.bias, self.strides, _triple(0), self.dilation, self.groups)
 
     def forward(self, *x):
         x = enforce_singleton(x)
@@ -918,7 +924,7 @@ class DepthwiseConv2d(Layer):
     def conv2d_forward(self, x):
         if self.padding_mode == 'circular':
             expanded_padding = ((self.padding[1] + 1) // 2, self.padding[1] // 2, (self.padding[0] + 1) // 2, self.padding[0] // 2)
-            x = F.pad(input, expanded_padding, mode='circular')
+            x = F.pad(x, expanded_padding, mode='circular')
         else:
             x = F.pad(x, (self.padding[1], self.padding[1], self.padding[0], self.padding[0]),mode='constant' if self.padding_mode == 'zero' else self.padding_mode)
 
@@ -960,10 +966,12 @@ class DepthwiseConv3d(Layer):
             self.to(self.device)
             self._built = True
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+    def get_padding(self, input_shape):
+        pad_w = 0
+        pad_h = 0
+        pad_z = 0
         if self.auto_pad == True:
-            iz, ih, iw = x.size()[-3:]
+            iz, ih, iw = list(input_shape)[-3:]
             kz, kh, kw = self.kernel_size[-3:]
             sz, sh, sw = self.strides[-3:]
             dz, dh, dw = self.dilation[-3:]
@@ -971,9 +979,32 @@ class DepthwiseConv3d(Layer):
             pad_z = max((oz - 1) * sz + (kz - 1) * dz + 1 - iz, 0)
             pad_h = max((oh - 1) * sh + (kh - 1) * dh + 1 - ih, 0)
             pad_w = max((ow - 1) * sw + (kw - 1) * dw + 1 - iw, 0)
-            if pad_z > 0 or pad_h > 0 or pad_w > 0:
-                x = F.pad(x, [pad_w // 2, pad_w - pad_w // 2, pad_h // 2, pad_h - pad_h // 2, pad_z // 2,
-                              pad_z - pad_z // 2], mode='constant' if self.padding_mode=='zero' else self.padding_mode)
+            if pad_h % 2 == 1 and sh > 1:
+                pad_h += 1
+            if pad_w % 2 == 1 and sw > 1:
+                pad_w += 1
+            if pad_z % 2 == 1 and sz > 1:
+                pad_z += 1
+        elif len(self.padding) == 3:
+            pad_z = self.padding[0] * 2
+            pad_h = self.padding[1] * 2
+            pad_w = self.padding[2] * 2
+
+        if self.padding_mode == 'circular':
+            self.padding = (
+            pad_z // 2, pad_z - (pad_z // 2), pad_h // 2, pad_h - (pad_h // 2), pad_w // 2, pad_w - (pad_w // 2))
+        else:
+            self.padding = (pad_z // 2, pad_h // 2, pad_w // 2)
+
+    def forward(self, *x):
+        x = enforce_singleton(x)
+        if self.padding_mode == 'circular':
+            expanded_padding = ((self.padding[2] + 1) // 2, self.padding[2] // 2, (self.padding[1] + 1) // 2, self.padding[1] // 2,(self.padding[0] + 1) // 2, self.padding[0] // 2)
+            x = F.pad(x, expanded_padding, mode='circular')
+        else:
+            x = F.pad(x, (self.padding[2], self.padding[2], self.padding[1], self.padding[1], self.padding[0], self.padding[0]),
+                      mode='constant' if self.padding_mode == 'zero' else self.padding_mode)
+
         x = self.conv1(x)
         if self.activation is not None:
             x = self.activation(x)
@@ -1133,7 +1164,7 @@ class GcdConv1d(Layer):
 
             self.channel_kernal = 2 if self.crossgroup_fusion == True and self.groups > 3 else 1
             self.channel_dilation = 1
-            if self.crossgroup_fusion == True and self.groups > 4:
+            if self.crossgroup_fusion == True and self.groups >= 4 :
                 self.channel_dilation = 2
             self.kernel_size = (self.channel_kernal,) + _pair(self.kernel_size)
             self.dilation = (self.channel_dilation,) + _pair(self.dilation)
@@ -1222,8 +1253,11 @@ class GcdConv2d(Layer):
         self.padding_mode = padding_mode
         self.activation = get_activation(activation)
         self.dilation = _pair(dilation)
-        self.self_norm = self_norm
         self.norm = None
+        if self_norm == True:
+            self.self_norm = self_norm
+            self.norm = get_normalization('instance')
+
         self.is_shuffle = is_shuffle
         self.use_bias = use_bias
         self.divisor_rank = divisor_rank
@@ -1232,9 +1266,6 @@ class GcdConv2d(Layer):
         self.groups = 1
 
         self.crossgroup_fusion = crossgroup_fusion
-
-        self.weight = None
-        self.bias = None
         self._built = False
 
     def calculate_gcd(self):
@@ -1254,32 +1285,41 @@ class GcdConv2d(Layer):
             self.groups = gcd_list[min(int(self.divisor_rank + 1), len(gcd_list))]
 
     def get_padding(self, input_shape):
-        pad_h = 0
         pad_w = 0
+        pad_h = 0
+        pad_z = 0
         if self.auto_pad == True:
-            ih, iw = to_list(input_shape)[-2:]
-            kh, kw = self.kernel_size[-2:]
-            sh, sw = self.strides[-2:]
-            dh, dw = self.dilation[-2:]
-            oh, ow = math.ceil(ih / sh), math.ceil(iw / sw)
-            pad_h = max(round((oh - 1) * sh + (kh - 1) * dh + 1 - ih), 0)
-            pad_w = max(round((ow - 1) * sw + (kw - 1) * dw + 1 - iw), 0)
+            iz, ih, iw = list(input_shape)[-3:]
+            kz, kh, kw = self.actual_kernel_size[-3:]
+            sz, sh, sw = self.actual_strides[-3:]
+            dz, dh, dw = self.actual_dilation[-3:]
+            oz, oh, ow = math.ceil(iz / sz), math.ceil(ih / sh), math.ceil(iw / sw)
+            pad_z = max((oz - 1) * sz + (kz - 1) * dz + 1 - iz, 0)
+            pad_h = max((oh - 1) * sh + (kh - 1) * dh + 1 - ih, 0)
+            pad_w = max((ow - 1) * sw + (kw - 1) * dw + 1 - iw, 0)
             if pad_h % 2 == 1 and sh > 1:
                 pad_h += 1
             if pad_w % 2 == 1 and sw > 1:
                 pad_w += 1
-        elif self.padding != _pair(0):
-            if len(self.padding) == 2:
-                pad_h = self.padding[0] * 2
-                pad_w = self.padding[1] * 2
+
+        elif len(self.padding) == 3:
+            pad_z = self.padding[0]
+            pad_h = self.padding[1] * 2
+            pad_w = self.padding[2] * 2
+
         if self.padding_mode == 'circular':
-            self.padding = ((pad_h + 1) // 2, pad_h // 2, (pad_w + 1) // 2, pad_w // 2)
+            self.padding = (pad_z , 0, pad_h // 2, pad_h - (pad_h // 2), pad_w // 2, pad_w - (pad_w // 2))
         else:
-            self.padding = (pad_h // 2, pad_h - pad_h // 2, pad_w // 2, pad_w - pad_w // 2)
+            self.padding = (pad_z , pad_h // 2, pad_w // 2)
 
     def build(self, input_shape):
         if self._built == False:
+            self.input_filters = input_shape[0].item()
             self.calculate_gcd()
+            if self.num_filters is None and self.depth_multiplier is not None:
+                self.num_filters = int(round(self.input_filters * self.depth_multiplier, 0))
+            if self.input_filters % self.groups != 0:
+                raise ValueError('in_channels must be divisible by groups')
             print('{0} input:{1} -> output:{2}   {3}  {4}  gcd:{5} group:{6}   通道縮放倍數:{7} '.format(self.name,
                                                                                                    self.input_filters,
                                                                                                    self.num_filters,
@@ -1289,48 +1329,44 @@ class GcdConv2d(Layer):
                                                                                                    self.groups,
                                                                                                    self.num_filters / float(
                                                                                                        self.input_filters)))
-            self.get_padding([self.input_filters // self.groups, self.groups, self.input_shape[1], self.input_shape[2]])
 
-            self.register_buffer('channel_kernal', torch.zeros((1)))
-            self.register_buffer('channel_dilation', torch.zeros((1)))
-            self.channel_kernal = torch.tensor(2 if self.crossgroup_fusion == True and self.groups > 3 else 1)
+
+            self.channel_kernal = torch.tensor(2 if self.crossgroup_fusion == True and self.groups > 4 else 1)
             self.channel_dilation = torch.tensor(1)
-            if self.crossgroup_fusion == True and self.groups > 4:
-                self.channel_dilation = torch.tensor(2)
-
-            reshape_input_shape = [-1, self.input_filters // self.groups, self.groups, self.input_shape[1],
-                                   self.input_shape[2]]
+            # if self.crossgroup_fusion == True and self.groups > 6:
+            #     self.channel_dilation = torch.tensor(2)
 
             self.actual_kernel_size = (self.channel_kernal,) + _pair(self.kernel_size)
             self.actual_dilation = (self.channel_dilation,) + _pair(self.dilation)
             self.actual_strides = (1,) + _pair(self.strides)
-            self.weight = Parameter(torch.Tensor(self.num_filters // self.groups, self.input_filters // self.groups,
-                                                 *self.actual_kernel_size))  #
+
+            self.get_padding([input_shape[0]//self.groups, self.groups, input_shape[1], input_shape[2]])
+
+            self.weight = Parameter(torch.Tensor(self.num_filters // self.groups, self.input_filters // self.groups, *self.actual_kernel_size))  #
             init.kaiming_uniform_(self.weight, mode='fan_in')
-            self._parameters['weight'] = self.weight
+
             if self.use_bias:
                 self.bias = Parameter(torch.Tensor(self.num_filters // self.groups))
                 init.zeros_(self.bias)
-                self._parameters['bias'] = self.bias
-
-            if self.self_norm == True:
-                self.norm = get_normalization('batch')
+            else:
+                self.register_parameter('bias',None)
 
             self.to(self.device)
             self._built = True
 
     def forward(self, *x):
         x = enforce_singleton(x)
-        x = F.pad(x, self.padding, mode='constant' if self.padding_mode=='zero' else self.padding_mode)
-        x = x.view(x.size(0), x.size(1) // self.groups, self.groups, x.size(2), x.size(3))
-        if self.crossgroup_fusion == True:
-            pad_g = max((self.groups - 1) * 1 + (self.channel_kernal - 1) * self.channel_dilation + 1 - self.groups, 0)
-            x = F.pad(x, [0, 0, 0, 0, pad_g // 2, pad_g - pad_g // 2], mode='circular')
+        x=x.view(x.size(0), x.size(1) // self.groups, self.groups, x.size(2), x.size(3))
+        x = F.pad(x, (self.padding[2], self.padding[2], self.padding[1], self.padding[1],0,0),mode='constant' if self.padding_mode == 'zero' else self.padding_mode)
+
+        if self.channel_kernal.item() == 2:
+            x=torch.cat([x,x[:,:,0:1,:,:]],dim=2)
+
 
         x = F.conv3d(x, self.weight, self.bias, self.actual_strides, _triple(0), self.actual_dilation, 1)
-        if self.is_shuffle == True:
-            x = x.transpose([2, 1])
 
+        if self.is_shuffle == True:
+            x = x.transpose_(2, 1)
         x = x.view(x.size(0), x.size(1) * x.size(2), x.size(3), x.size(4))
         if self.self_norm == True:
             x = self.norm(x)
@@ -1416,14 +1452,11 @@ class GcdConv2d_1(Layer):
                 pad_h += 1
             if pad_w % 2 == 1 and sw > 1:
                 pad_w += 1
-        elif self.padding != _pair(0):
-            if len(self.padding) == 2:
-                pad_h = self.padding[0] * 2
-                pad_w = self.padding[1] * 2
-        if self.padding_mode == 'circular':
-            self.padding = ((pad_h + 1) // 2, pad_h // 2, (pad_w + 1) // 2, pad_w // 2)
-        else:
-            self.padding = (pad_h // 2, pad_h - pad_h // 2, pad_w // 2, pad_w - pad_w // 2)
+        elif len(self.padding) == 2:
+            pad_h = self.padding[0] * 2
+            pad_w = self.padding[1] * 2
+
+        self.padding = (pad_h // 2, pad_w // 2)
 
     def build(self, input_shape):
         if self._built == False:
@@ -1456,7 +1489,14 @@ class GcdConv2d_1(Layer):
 
     def forward(self, *x):
         x = enforce_singleton(x)
-        x = F.pad(x, self.padding, mode='constant' if self.padding_mode=='zero' else self.padding_mode)
+        if self.padding_mode == 'circular':
+            expanded_padding = (
+            (self.padding[1] + 1) // 2, self.padding[1] // 2, (self.padding[0] + 1) // 2, self.padding[0] // 2)
+            x = F.pad(input, expanded_padding, mode='circular')
+        else:
+            x = F.pad(x, (self.padding[1], self.padding[1], self.padding[0], self.padding[0]),
+                      mode='constant' if self.padding_mode == 'zero' else self.padding_mode)
+
         x = F.conv2d(x, self.weight, self.bias, self.strides, self.padding, self.dilation, self.groups)
         if self.self_norm == True:
             x = self.norm(x)
