@@ -22,7 +22,8 @@ from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
 
 import numpy as np
 
-__all__ = ['MaxPool2d','MaxPool1d','MaxPool3d','MaxUnpool1d','MaxUnpool2d','MaxUnpool3d','AvgPool1d','AvgPool2d','AvgPool3d','GlobalAvgPool2d','AdaptiveAvgPool2d']
+__all__ = ['MaxPool2d', 'MaxPool1d', 'MaxPool3d', 'MaxUnpool1d', 'MaxUnpool2d', 'MaxUnpool3d', 'AvgPool1d', 'AvgPool2d',
+           'AvgPool3d', 'GlobalAvgPool2d', 'AdaptiveAvgPool2d']
 
 _session = get_session()
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
@@ -45,29 +46,30 @@ _quadruple = _ntuple(4)
 
 
 class _PoolNd(Layer):
-    __constants__ = ['kernel_size',  'strides', 'auto_pad','padding','dilation','ceil_mode']
+    __constants__ = ['kernel_size', 'strides', 'auto_pad', 'padding', 'dilation', 'ceil_mode']
 
-    def __init__(self, kernel_size, strides=None, auto_pad=True,padding_mode='replicate', dilation=1,name='', **kwargs):
+    def __init__(self, kernel_size, strides=None, auto_pad=True, padding_mode='replicate', dilation=1, name='',
+                 **kwargs):
         super(_PoolNd, self).__init__()
         self.kernel_size = kernel_size
         self.strides = strides or kernel_size
-        self.auto_pad=auto_pad
-        self.padding_mode=padding_mode
+        self.auto_pad = auto_pad
+        self.padding_mode = padding_mode
         self.padding = 0
         self.dilation = dilation
-        self.return_indices =kwargs.get('return_indices',False)
-        self.ceil_mode = kwargs.get('ceil_mode',False)
-        self.count_include_pad= kwargs.get('count_include_pad',False)
-        self.name=name
+        self.return_indices = kwargs.get('return_indices', False)
+        self.ceil_mode = kwargs.get('ceil_mode', False)
+        self.count_include_pad = kwargs.get('count_include_pad', False)
+        self.name = name
 
     def build(self, input_shape):
-        self._built = True
+        if self._built == False:
+            self.get_padding(input_shape)
+            self._built = True
+
     def extra_repr(self):
         return 'kernel_size={kernel_size}, strides={strides}, padding={padding}' \
-            ', dilation={dilation}'.format(**self.__dict__)
-
-
-
+               ', dilation={dilation}'.format(**self.__dict__)
 
 
 class MaxPool1d(_PoolNd):
@@ -113,17 +115,19 @@ class MaxPool1d(_PoolNd):
         https://github.com/vdumoulin/conv_arithmetic/blob/master/README.md
     """
 
-    def __init__(self, kernel_size, strides=None, auto_pad=True,padding_mode='replicate',name='', **kwargs):
-        super(MaxPool1d, self).__init__(kernel_size, strides, auto_pad,1,name, **kwargs)
+    def __init__(self, kernel_size, strides=None, auto_pad=True, padding_mode='replicate', name='', **kwargs):
+        super(MaxPool1d, self).__init__(kernel_size, strides, auto_pad, 1, name, **kwargs)
         self.kernel_size = _single(kernel_size)
         self.strides = _single(strides if strides is not None else kernel_size)
         self.padding = _single(self.padding)
-        self.padding_mode=padding_mode
+        self.padding_mode = padding_mode
         self.ceil_mode = kwargs.get('ceil_mode', False)
         self.return_indices = kwargs.get('return_indices', False)
+
     def forward(self, *x):
         x = enforce_singleton(x)
-        return F.max_pool1d(x, self.kernel_size, self.strides, self.padding, self.dilation, self.ceil_mode, self.return_indices )
+        return F.max_pool1d(x, self.kernel_size, self.strides, self.padding, self.dilation, self.ceil_mode,
+                            self.return_indices)
 
 
 class MaxPool2d(_PoolNd):
@@ -184,32 +188,49 @@ class MaxPool2d(_PoolNd):
     .. _link:
         https://github.com/vdumoulin/conv_arithmetic/blob/master/README.md
     """
-    def __init__(self, kernel_size, strides=None, auto_pad=True,padding_mode='zero', name='',**kwargs):
-        super(MaxPool2d, self).__init__(kernel_size, strides , auto_pad, padding_mode,1,name,**kwargs)
+
+    def __init__(self, kernel_size, strides=None, auto_pad=True, padding_mode='zero', name='', **kwargs):
+        super(MaxPool2d, self).__init__(kernel_size, strides, auto_pad, padding_mode, 1, name, **kwargs)
         self.kernel_size = _pair(kernel_size)
         self.strides = _pair(strides if strides is not None else kernel_size)
         self.padding = _pair(0)
-        if padding_mode=='zero':
-            self.padding_mode='constant'
+        if padding_mode == 'zero':
+            self.padding_mode = 'constant'
         else:
             self.padding_mode = padding_mode
         self.ceil_mode = kwargs.get('ceil_mode', False)
         self.return_indices = kwargs.get('return_indices', False)
-    def forward(self, *x):
-        x=enforce_singleton(x)
+
+    def get_padding(self, input_shape):
+        pad_h = 0
+        pad_w = 0
         if self.auto_pad == True:
-            ih, iw =  list(x.size())[-2:]
+            ih, iw = input_shape[-2:]
             kh, kw = self.kernel_size[-2:]
             sh, sw = self.strides[-2:]
 
             oh, ow = math.ceil(ih / sh), math.ceil(iw / sw)
-            pad_h = max((oh - 1) * sh + (kh - 1)  + 1 - ih, 0)
+            pad_h = max((oh - 1) * sh + (kh - 1) + 1 - ih, 0)
             pad_w = max((ow - 1) * sw + (kw - 1) + 1 - iw, 0)
-            if pad_h > 0 or pad_w > 0:
-                x = F.pad(x, [pad_w // 2, pad_w - pad_w // 2, pad_h // 2, pad_h - pad_h // 2], mode=self.padding_mode)
-        return F.max_pool2d(x, self.kernel_size, self.strides,
-                            self.padding, self.dilation, self.ceil_mode,
-                            self.return_indices )
+            if pad_h % 2 == 1 and sh > 1:
+                pad_h += 1
+            if pad_w % 2 == 1 and sw > 1:
+                pad_w += 1
+
+        elif len(self.padding) == 2:
+            pad_h = self.padding[0] * 2
+            pad_w = self.padding[1] * 2
+
+        self.padding = (int(pad_h / 2), int(pad_w / 2))
+
+    def forward(self, *x):
+        x = enforce_singleton(x)
+        # self.get_padding(x.size()[1:])
+        # if self.padding[0] > 0 or self.padding[1] > 0:
+        #     x = F.pad(x, (self.padding[1] , self.padding[1], self.padding[0] , self.padding[0] ), mode='constant'
+        #     if self.padding_mode == 'zero' else self.padding_mode)
+        return F.max_pool2d(x, self.kernel_size, self.strides, self.padding, self.dilation, self.ceil_mode,
+                            self.return_indices)
 
 
 class MaxPool3d(_PoolNd):
@@ -222,9 +243,11 @@ class MaxPool3d(_PoolNd):
 
     .. math::
         \begin{aligned}
-            \text{out}(N_i, C_j, d, h, w) ={} & \max_{k=0, \ldots, kD-1} \max_{m=0, \ldots, kH-1} \max_{n=0, \ldots, kW-1} \\
+            \text{out}(N_i, C_j, d, h, w) ={} & \max_{k=0, \ldots, kD-1} \max_{m=0, \ldots, kH-1} \max_{n=0, \ldots, 
+            kW-1} \\
                                               & \text{input}(N_i, C_j, \text{stride[0]} \times d + k,
-                                                             \text{stride[1]} \times h + m, \text{stride[2]} \times w + n)
+                                                             \text{stride[1]} \times h + m, \text{stride[2]} \times w 
+                                                             + n)
         \end{aligned}
 
     If :attr:`padding` is non-zero, then the input is implicitly zero-padded on both sides
@@ -274,17 +297,18 @@ class MaxPool3d(_PoolNd):
     .. _link:
         https://github.com/vdumoulin/conv_arithmetic/blob/master/README.md
     """  # noqa: E501
-    def __init__(self, kernel_size, strides=None, auto_pad=True, name='',**kwargs):
-        super(MaxPool3d, self).__init__(kernel_size, strides, auto_pad, 1,name,**kwargs)
+
+    def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
+        super(MaxPool3d, self).__init__(kernel_size, strides, auto_pad, 1, name, **kwargs)
         self.kernel_size = _triple(kernel_size)
         self.strides = _triple(strides if strides is not None else kernel_size)
         self.padding = _triple(self.padding)
         self.ceil_mode = kwargs.get('ceil_mode', False)
-        self.return_indices=kwargs.get('return_indices',False)
+        self.return_indices = kwargs.get('return_indices', False)
+
     def forward(self, input):
-        return F.max_pool3d(input, self.kernel_size, self.strides,
-                            self.padding, self.dilation, self.ceil_mode,
-                            self.return_indices )
+        return F.max_pool3d(input, self.kernel_size, self.strides, self.padding, self.dilation, self.ceil_mode,
+                            self.return_indices)
 
 
 class MaxUnpool1d(_PoolNd):
@@ -341,15 +365,14 @@ class MaxUnpool1d(_PoolNd):
         tensor([[[ 0.,  2.,  0.,  4.,  0.,  6.,  0., 8.]]])
     """
 
-    def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs ):
-        super(MaxUnpool1d, self).__init__(kernel_size, strides, auto_pad,1,name, **kwargs)
+    def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
+        super(MaxUnpool1d, self).__init__(kernel_size, strides, auto_pad, 1, name, **kwargs)
         self.kernel_size = _single(kernel_size)
         self.strides = _single(strides if strides is not None else kernel_size)
         self.padding = _single(self.padding)
 
     def forward(self, x, indices, output_size=None):
-        return F.max_unpool1d(x, indices, self.kernel_size, self.strides,
-                              self.padding, output_size)
+        return F.max_unpool1d(x, indices, self.kernel_size, self.strides, self.padding, output_size)
 
 
 class MaxUnpool2d(_PoolNd):
@@ -414,8 +437,8 @@ class MaxUnpool2d(_PoolNd):
                   [  0.,   0.,   0.,   0.,   0.]]]])
     """
 
-    def __init__(self, kernel_size, strides=None, auto_pad=True,name='', **kwargs):
-        super(MaxUnpool2d, self).__init__(kernel_size, strides, auto_pad,1,name, **kwargs)
+    def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
+        super(MaxUnpool2d, self).__init__(kernel_size, strides, auto_pad, 1, name, **kwargs)
         self.kernel_size = _pair(kernel_size)
         self.strides = _pair(strides or kernel_size)
         self.auto_pad = auto_pad
@@ -423,7 +446,7 @@ class MaxUnpool2d(_PoolNd):
 
     def forward(self, x, indices, output_size=None):
         return F.max_unpool2d(x, indices, self.kernel_size, self.strides,
-                              self.padding, output_size)
+                              (self.padding[1], self.padding[1], self.padding[0], self.padding[0]), output_size)
 
 
 class MaxUnpool3d(_PoolNd):
@@ -478,15 +501,13 @@ class MaxUnpool3d(_PoolNd):
     """
 
     def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
-        super(MaxUnpool3d, self).__init__(self, kernel_size, strides, auto_pad,1,name, **kwargs)
+        super(MaxUnpool3d, self).__init__(self, kernel_size, strides, auto_pad, 1, name, **kwargs)
         self.kernel_size = _triple(kernel_size)
         self.strides = _triple(strides if strides is not None else kernel_size)
         self.padding = _triple(self.padding)
 
     def forward(self, x, indices, output_size=None):
-        return F.max_unpool3d(x, indices, self.kernel_size, self.strides,
-                              self.padding, output_size)
-
+        return F.max_unpool3d(x, indices, self.kernel_size, self.strides, self.padding, output_size)
 
 
 class AvgPool1d(_PoolNd):
@@ -530,19 +551,17 @@ class AvgPool1d(_PoolNd):
         >>> m(torch.tensor([[[1.,2,3,4,5,6,7]]]))
         tensor([[[ 2.,  4.,  6.]]])
     """
-    def __init__(self, kernel_size, strides=None, auto_pad=True, name='',  **kwargs ):
-        super(AvgPool1d, self).__init__(kernel_size, strides, auto_pad,1,name, **kwargs)
+
+    def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
+        super(AvgPool1d, self).__init__(kernel_size, strides, auto_pad, 1, name, **kwargs)
         self.kernel_size = _single(kernel_size)
         self.strides = _single(strides if strides is not None else kernel_size)
         self.padding = _single(self.padding)
-        self.ceil_mode = kwargs.get('ceil_mode',False)
-        self.count_include_pad =kwargs.get('count_include_pad',False)
-
+        self.ceil_mode = kwargs.get('ceil_mode', False)
+        self.count_include_pad = kwargs.get('count_include_pad', False)
 
     def forward(self, input):
-        return F.avg_pool1d(
-            input, self.kernel_size, self.strides, self.padding, self.ceil_mode,
-            self.count_include_pad)
+        return F.avg_pool1d(input, self.kernel_size, self.strides, self.padding, self.ceil_mode, self.count_include_pad)
 
 
 class AvgPool2d(_PoolNd):
@@ -597,21 +616,44 @@ class AvgPool2d(_PoolNd):
         >>> output = m(input)
     """
     __constants__ = ['kernel_size', 'stride', 'padding', 'ceil_mode', 'count_include_pad', 'divisor_override']
-    def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
-        super(AvgPool2d, self).__init__( kernel_size, strides, auto_pad,1,name, **kwargs)
-        self.kernel_size =kernel_size
-        self.strides = strides if strides is not None else kernel_size
+
+    def __init__(self, kernel_size, strides=None, auto_pad=True,count_include_pad=True, divisor_override=None, name='', **kwargs):
+        super(AvgPool2d, self).__init__(kernel_size, strides, auto_pad, 1, name, **kwargs)
+        self.kernel_size = _pair(kernel_size)
+        self.strides = _pair(strides if strides is not None else kernel_size)
+        self.auto_pad == auto_pad
+
+        self.ceil_mode = kwargs.get('ceil_mode', False)
+        self.count_include_pad = kwargs.get('count_include_pad', True)
+        self.divisor_override = kwargs.get('divisor_override', None)
+
+    def get_padding(self, input_shape):
+        pad_h = 0
+        pad_w = 0
         if self.auto_pad == True:
-            self.padding=_pair(1)
+            ih, iw = input_shape[-2:]
+            kh, kw = self.kernel_size[-2:]
+            sh, sw = self.strides[-2:]
+
+            oh, ow = math.ceil(ih / sh), math.ceil(iw / sw)
+            pad_h = max((oh - 1) * sh + (kh - 1) + 1 - ih, 0)
+            pad_w = max((ow - 1) * sw + (kw - 1) + 1 - iw, 0)
+            if pad_h % 2 == 1 and sh > 1:
+                pad_h += 1
+            if pad_w % 2 == 1 and sw > 1:
+                pad_w += 1
+
+        elif len(self.padding) == 2:
+            pad_h = self.padding[0] * 2
+            pad_w = self.padding[1] * 2
+
+        self.padding = (int(pad_h / 2), int(pad_w / 2))
 
-        self.ceil_mode = kwargs.get('ceil_mode',False)
-        self.count_include_pad =kwargs.get('count_include_pad',False)
-        self.divisor_override  =kwargs.get('divisor_override',None)
     def forward(self, *x):
-        x=enforce_singleton(x)
+        x = enforce_singleton(x)
 
-        return F.avg_pool2d(x, self.kernel_size, self.strides,
-                            self.padding, self.ceil_mode, self.count_include_pad, self.divisor_override)
+        return F.avg_pool2d(x, self.kernel_size, self.strides, self.padding, self.ceil_mode, self.count_include_pad,
+                            self.divisor_override)
 
 
 class AvgPool3d(_PoolNd):
@@ -674,19 +716,18 @@ class AvgPool3d(_PoolNd):
     """
     __constants__ = ['kernel_size', 'stride', 'padding', 'ceil_mode', 'count_include_pad', 'divisor_override']
 
-
-
     def __init__(self, kernel_size, strides=None, auto_pad=True, name='', **kwargs):
-        super(AvgPool3d, self).__init__( kernel_size, strides, auto_pad,1,name, **kwargs)
+        super(AvgPool3d, self).__init__(kernel_size, strides, auto_pad, 1, name, **kwargs)
         self.kernel_size = _triple(kernel_size)
         self.strides = _triple(strides if strides is not None else kernel_size)
         self.padding = _triple(self.padding)
-        self.ceil_mode = kwargs.get('ceil_mode',False)
-        self.count_include_pad =kwargs.get('count_include_pad',False)
-        self.divisor_override  =kwargs.get('divisor_override',None)
+        self.ceil_mode = kwargs.get('ceil_mode', False)
+        self.count_include_pad = kwargs.get('count_include_pad', False)
+        self.divisor_override = kwargs.get('divisor_override', None)
+
     def forward(self, input):
-        return F.avg_pool3d(input, self.kernel_size, self.strides,
-                            self.padding, self.ceil_mode, self.count_include_pad, self.divisor_override)
+        return F.avg_pool3d(input, self.kernel_size, self.strides, self.padding, self.ceil_mode, self.count_include_pad,
+                            self.divisor_override)
 
     def __setstate__(self, d):
         super(AvgPool3d, self).__setstate__(d)
@@ -696,37 +737,42 @@ class AvgPool3d(_PoolNd):
 
 
 class GlobalAvgPool2d(Layer):
-    def __init__(self,keepdim=False,name='avg_pool'):
+    def __init__(self, keepdim=False, name='avg_pool'):
         super(GlobalAvgPool2d, self).__init__()
-        self.keepdim=keepdim
-        self.name=name
-    def forward(self,*x):
-        x=enforce_singleton(x)
-        if x.size(1)==1:
+        self.keepdim = keepdim
+        self.name = name
+
+    def build(self, input_shape):
+        if self._built == False:
+            if self.keepdim == True:
+                output_shape = input_shape.clone()
+                output_shape[1] = 1
+                output_shape[2] = 1
+                self.output_shape = output_shape
+            else:
+                self.output_shape = input_shape[0]
+            self._built = True
+
+    def forward(self, *x):
+        x = enforce_singleton(x)
+        if x.size(1) == 1:
             x = x.view(x.size(0), x.size(1), -1).mean(dim=-1, keepdim=self.keepdim)
-            if self.keepdim==False:
-                x= x.squeeze()
+
         else:
-            x=x.view(x.size(0),x.size(1),-1).mean(dim=-1,keepdim=self.keepdim)
+            x = x.view(x.size(0), x.size(1), -1).mean(dim=-1, keepdim=self.keepdim)
         return x
 
 
-
-
 class AdaptiveAvgPool2d(Layer):
-    def __init__(self,output_size,name='adaptive_avg_pool'):
+    def __init__(self, output_size, name='adaptive_avg_pool'):
         super(AdaptiveAvgPool2d, self).__init__()
-        self.output_size=_pair(output_size)
-        self.name=name
-    def forward(self,*x):
-        x=enforce_singleton(x)
+        self.output_size = _pair(output_size)
+        self.name = name
+
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return F.adaptive_avg_pool2d(x, self.output_size)
 
-
-
-
-
-
 #
 # class GlobalAvgPool1d(Layer):
 #     def __init__(self):
