@@ -39,11 +39,14 @@ from .tensorflow_layers import *
 from itertools import repeat
 import inspect
 
-from ..backend.common import *
-from ..backend.tensorflow_backend import *
+
 from tensorflow.python.training.tracking import base as trackable
 from tensorflow.python.keras.engine import base_layer
 
+from ..backend.common import *
+from ..backend.tensorflow_backend import *
+from ..layers.tensorflow_layers import *
+
 _tf_data_format = 'channels_last'
 
 __all__ = ['Conv2d_Block', 'TransConv2d_Block', 'ShortCut2d']
@@ -79,7 +82,7 @@ def get_layer_repr(layer):
     return main_str
 
 
-class Conv1d_Block(Sequential):
+class Conv1d_Block(tf.keras.Sequential):
     def __init__(self, kernel_size=(3), num_filters=32, strides=1, input_shape=None, auto_pad=True,
                  activation='leaky_relu', normalization=None, use_bias=False, dilation=1, groups=1, add_noise=False,
                  noise_intensity=0.001, dropout_rate=0, name='', **kwargs):
@@ -103,7 +106,7 @@ class Conv1d_Block(Sequential):
             self.add(self.drop)
 
 
-class Conv2d_Block(Sequential):
+class Conv2d_Block(tf.keras.Sequential):
     def __init__(self, kernel_size=(3, 3), num_filters=32, strides=1, input_shape=None, auto_pad=True,
                  activation='leaky_relu', normalization=None, use_bias=False, dilation=1, groups=1, add_noise=False,
                  noise_intensity=0.001, dropout_rate=0, name='', **kwargs):
@@ -166,7 +169,7 @@ class Conv2d_Block(Sequential):
 #         base_config = super(Conv2d_Block1, self).get_config()
 #         return dict(list(base_config.items()) + list(config.items()))
 
-class Conv3d_Block(Sequential):
+class Conv3d_Block(tf.keras.Sequential):
     def __init__(self, kernel_size=(3, 3, 3), num_filters=32, strides=1, input_shape=None, auto_pad=True,
                  activation='leaky_relu', normalization=None, use_bias=False, dilation=1, groups=1, add_noise=False,
                  noise_intensity=0.001, dropout_rate=0, name='', **kwargs):
@@ -233,7 +236,7 @@ class Conv3d_Block(Sequential):
 #         return get_layer_repr(self)
 
 
-class TransConv2d_Block(Sequential):
+class TransConv2d_Block(tf.keras.Sequential):
     def __init__(self, kernel_size=(3, 3), num_filters=32, strides=1, auto_pad=True, activation='leaky_relu',
                  normalization=None, use_bias=False, dilation=1, groups=1, add_noise=False, noise_intensity=0.001,
                  dropout_rate=0, name='', **kwargs):
@@ -271,7 +274,7 @@ class TransConv2d_Block(Sequential):
         return dict(list(base_config.items()) + list(config.items()))
 
 
-class TransConv3d_Block(Sequential):
+class TransConv3d_Block(tf.keras.Sequential):
     def __init__(self, kernel_size=(3, 3, 3), num_filters=32, strides=1, input_shape=None, auto_pad=True,
                  activation='leaky_relu', normalization=None, use_bias=False, dilation=1, groups=1, add_noise=False,
                  noise_intensity=0.001, dropout_rate=0, name='', **kwargs):
@@ -304,7 +307,7 @@ class TransConv3d_Block(Sequential):
         self._conv = value
 
 
-class Classifer1d(Sequential):
+class Classifer1d(tf.keras.Sequential):
     def __init__(self, num_classes=10, is_multilable=False, classifier_type=ClassfierType.dense, name='', **kwargs):
         super(Classifer1d, self).__init__(name=name)
         self.classifier_type = classifier_type
