@@ -1,14 +1,10 @@
-import platform
-import tkinter
+import tkinter as tk
 from .ipython_utils import is_in_ipython,is_in_colab
 if is_in_ipython():
     from IPython import display
+import matplotlib
+matplotlib.use('Qt5Agg' if not is_in_ipython() and not is_in_colab() else 'TkAgg' if not is_in_colab() else 'NbAgg' )
 
-if not is_in_colab:
-    import matplotlib
-    matplotlib.use('Qt5Agg' if not is_in_ipython() else 'TkAgg')
-else:
-    import matplotlib
 import matplotlib.pyplot as plt
 from matplotlib.collections import PolyCollection
 
@@ -24,7 +20,6 @@ __all__ = ['tile_rgb_images', 'loss_metric_curve', 'steps_histogram']
 def tile_rgb_images(*imgs, row=3, save_path=None, imshow=False):
     make_dir_if_need(save_path)
     suffix = get_time_suffix()
-    plt.ioff()
     if len(imgs)==1 and row==1:
         img = array2image(imgs[0][0])
         filename = save_path.format(suffix)
@@ -33,16 +28,18 @@ def tile_rgb_images(*imgs, row=3, save_path=None, imshow=False):
         if imshow:
             if is_in_ipython():
                 plt.axis("off")
+                plt.ioff()
                 display.display(plt.gcf())
-                plt.close()
             else:
                 plt.axis("off")
                 plt.imshow(img, interpolation="nearest", animated=True)
+                plt.ioff()
                 plt.gcf().show()
     else:
         fig = plt.gcf()
         fig.set_size_inches(len(imgs) * 2, row * 2)
         plt.clf()
+        plt.ion()  # is not None:
 
 
         for m in range(row * len(imgs)):
@@ -56,9 +53,10 @@ def tile_rgb_images(*imgs, row=3, save_path=None, imshow=False):
             plSize = fig.get_size_inches()
             fig.set_size_inches((int(round(plSize[0] * 0.75, 0)), int(round(plSize[1] * 0.75, 0))))
             if is_in_ipython():
+                plt.ioff()
                 display.display(plt.gcf())
-                plt.close(fig)
             else:
+                plt.ioff()
                 plt.gcf().show()
 
 
@@ -67,7 +65,7 @@ def loss_metric_curve(losses, metrics, sample_collected=None, legend=None, calcu
     fig = plt.gcf()
     fig.set_size_inches(18, 8)
     plt.clf()
-    plt.ioff()
+    plt.ion()  # is not None:
     collected_samples = []
     if sample_collected is not None and len(sample_collected) > 0:
         sample_collected = np.array(sample_collected)
@@ -112,13 +110,13 @@ def loss_metric_curve(losses, metrics, sample_collected=None, legend=None, calcu
         for i in range(len(metrics)):
             item = metrics[i]
             for k, v in item.items():
-                if len(collected_samples) > 0:
+                if len(collected_samples) ==len(v)> 0:
                     plt.plot(collected_samples, np.array(item[k]))
-                else:
+                elif len(v)> 0:
                     plt.plot(np.array(item[k]))
-                if legend is not None:
+                if len(v)> 0 and legend is not None:
                     legend_list.append(['{0} {1}'.format(k, legend[i])])
-                else:
+                elif len(v)> 0:
                     legend_list.append(['{0} {1}'.format(k, i)])
         plt.legend(legend_list, loc='upper left')
 
@@ -134,9 +132,10 @@ def loss_metric_curve(losses, metrics, sample_collected=None, legend=None, calcu
         plt.savefig(save_path, bbox_inches='tight')
     if imshow == True:
         if is_in_ipython():
+            plt.ioff()
             display.display(plt.gcf())
-            plt.close(fig)
         else:
+            plt.ioff()
             plt.show(block=False)
 
 
@@ -167,7 +166,7 @@ def steps_histogram(grads, weights, sample_collected=None, bins=None, size=(18,
         sample = np.arange(len(sample_collected))
         collected_samples = sample[sample_collected == 1]
 
-    plt.ioff()
+    plt.ion()
     fig = plt.figure(figsize=size)
     fig.patch.set_facecolor('white')
     ax = fig.add_subplot(1, 2, 1, projection='3d')
@@ -238,15 +237,16 @@ def steps_histogram(grads, weights, sample_collected=None, bins=None, size=(18,
     if save_path is not None:
         plt.savefig(save_path, bbox_inches='tight')
     if imshow == True:
-        if is_in_ipython():
+        if is_in_ipython() or is_in_colab():
             display.display(plt.gcf())
             plt.close(fig)
         else:
+            plt.ioff()
             plt.show(block=False)
 
 
-def centerloss_plot(feat, labels,imshow=True):
-    plt.ioff()
+def centerloss_plot(feat, labels):
+    plt.ion()
     c = ['#ff0000', '#ffff00', '#00ff00', '#00ffff', '#0000ff', '#ff00ff', '#990000', '#999900', '#009900', '#009999']
     plt.clf()
     for i in range(10):
@@ -255,12 +255,7 @@ def centerloss_plot(feat, labels,imshow=True):
     plt.xlim(xmin=-8, xmax=8)
     plt.ylim(ymin=-8, ymax=8)
     plt.draw()
-    if imshow == True:
-        if is_in_ipython():
-            display.display(plt.gcf())
-            plt.close(fig)
-        else:
-            plt.show(block=False)
+    plt.pause(0.001)
 
 
 def plot_confusion_matrix(cm, class_names, figsize=(16, 16), normalize=False, title="Confusion matrix", fname=None,
@@ -268,6 +263,10 @@ def plot_confusion_matrix(cm, class_names, figsize=(16, 16), normalize=False, ti
     """Render the confusion matrix and return matplotlib's figure with it.
     Normalization can be applied by setting `normalize=True`.
     """
+    import matplotlib
+
+    matplotlib.use("Agg")
+    import matplotlib.pyplot as plt
 
     cmap = plt.cm.Oranges
 
