@@ -1,4 +1,3 @@
-
 from __future__ import division
 from __future__ import print_function
 import math
@@ -21,7 +20,7 @@ from itertools import repeat
 
 
 from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy,to_tensor,Layer,Sequential,Input,summary,get_signature
+from ..backend.pytorch_backend import to_numpy,to_tensor,Layer,Sequential,Input,summary
 from ..layers.pytorch_layers import *
 from ..layers.pytorch_activations import  get_activation,Identity,Relu
 from ..layers.pytorch_normalizations import get_normalization,BatchNorm2d
