@@ -20,7 +20,7 @@ from itertools import repeat
 
 
 from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy,to_tensor,Layer,Sequential,get_signature
+from ..backend.pytorch_backend import to_numpy,to_tensor,Layer,Sequential
 from ..layers.pytorch_layers import *
 from ..layers.pytorch_activations import  get_activation,Identity,Relu
 from ..layers.pytorch_normalizations import get_normalization
@@ -349,7 +349,7 @@ def EfficientNetB4(include_top=True,
     effb4 =EfficientNet(1.4, 1.8, 380, 0.4, model_name='efficientnet-b4',include_top=include_top,num_classes=classes)
     if pretrained==True:
         download_model_from_google_drive('1X4ZOBR_ETRHZJeffJHvCmWTTy9_aW8SP',dirname,'efficientnet-b4.pth')
-        recovery_model=torch.load(os.path.join(dirname,'efficientnet-b4.pth'))
+        recovery_model=torch.load(sanitize_path(os.path.join(dirname,'efficientnet-b4.pth')))
         recovery_model.input_shape = input_shape
         recovery_model.to(_device)
         if include_top==False:
