@@ -20,7 +20,7 @@ from torch._six import container_abcs
 from itertools import repeat
 
 from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential,get_signature
+from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
 from ..layers.pytorch_layers import *
 from ..layers.pytorch_activations import get_activation, Identity, LeakyRelu
 from ..layers.pytorch_normalizations import get_normalization, BatchNorm
