@@ -1,11 +1,6 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-import matplotlib
-
-from ..misc.ipython_utils import is_in_ipython
-if is_in_ipython():
-    from IPython import display
 import itertools
 from matplotlib.collections import PolyCollection
 import math
