@@ -2,9 +2,12 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 import math
+import sys
 import os
 import inspect
 import numpy as np
+from math import ceil
+from itertools import product as product
 from collections import *
 from functools import partial
 import uuid
@@ -20,7 +23,7 @@ from itertools import repeat
 
 from ..backend.pytorch_ops import *
 from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential,get_signature
+from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential
 from ..data.bbox_common import xywh_to_xyxy,xyxy_to_xywh
 from ..layers.pytorch_layers import *
 from ..layers.pytorch_activations import get_activation, Identity, Relu
@@ -65,6 +68,56 @@ feature_map_w_h_list = [[40, 20, 10, 5], [30, 15, 8, 4]]  # default feature map
 priors = []
 
 
+
+
+class PriorBox(object):
+    def __init__(self, cfg, image_size=None, phase='train'):
+        super(PriorBox, self).__init__()
+        #self.aspect_ratios = cfg['aspect_ratios']
+        self.min_sizes = cfg['min_sizes']
+        self.steps = cfg['steps']
+        self.clip = cfg['clip']
+        self.image_size = image_size
+        #self.feature_maps = [[ceil(self.image_size[0]/step), ceil(self.image_size[1]/step)] for step in self.steps]
+
+        for ii in range(4):
+            if(self.steps[ii] != pow(2,(ii+3))):
+                print("steps must be [8,16,32,64]")
+                sys.exit()
+
+        self.feature_map_2th = [int(int((self.image_size[0] + 1) / 2) / 2),
+                                int(int((self.image_size[1] + 1) / 2) / 2)]
+        self.feature_map_3th = [int(self.feature_map_2th[0] / 2),
+                                int(self.feature_map_2th[1] / 2)]
+        self.feature_map_4th = [int(self.feature_map_3th[0] / 2),
+                                int(self.feature_map_3th[1] / 2)]
+        self.feature_map_5th = [int(self.feature_map_4th[0] / 2),
+                                int(self.feature_map_4th[1] / 2)]
+        self.feature_map_6th = [int(self.feature_map_5th[0] / 2),
+                                int(self.feature_map_5th[1] / 2)]
+
+        self.feature_maps = [self.feature_map_3th, self.feature_map_4th,
+                             self.feature_map_5th, self.feature_map_6th]
+
+    def forward(self):
+        anchors = []
+        for k, f in enumerate(self.feature_maps):
+            min_sizes = self.min_sizes[k]
+            for i, j in product(range(f[0]), range(f[1])):
+                for min_size in min_sizes:
+                    s_kx = min_size / self.image_size[1]
+                    s_ky = min_size / self.image_size[0]
+
+                    cx = (j + 0.5) * self.steps[k] / self.image_size[1]
+                    cy = (i + 0.5) * self.steps[k] / self.image_size[0]
+                    anchors += [cx, cy, s_kx, s_ky]
+        # back to torch land
+        output = torch.Tensor(anchors).view(-1, 4)
+        if self.clip:
+            output.clamp_(max=1, min=0)
+        return output
+
+
 def generate_priors(feature_map_list, shrinkage_list, image_size, min_boxes, clamp=True) -> torch.Tensor:
     priors = []
     for index in range(0, len(feature_map_list[0])):
@@ -80,7 +133,7 @@ def generate_priors(feature_map_list, shrinkage_list, image_size, min_boxes, cla
                     h = min_box / image_size[1]
                     priors.append([x_center, y_center, w, h])
     print("priors nums:{}".format(len(priors)))
-    priors = to_tensor(priors)
+    priors = to_tensor(priors)#.view(-1, 4)
     if clamp:
         torch.clamp(priors, 0.0, 1.0, out=priors)
     return priors
@@ -137,11 +190,11 @@ def basic_rfb(num_filters, scale=0.1):
             Conv2d_Block((3, 3), depth_multiplier=1.34, strides=1, groups=1, auto_pad=True, use_bias=False,
                          activation='relu', normalization='batch'),
             Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, auto_pad=True, use_bias=False, dilation=5,
-                         activation=None, normalization='batch'), name='branch2'), mode=ShortcutMode.concate),
+                         activation=None, normalization='batch'), name='branch2'), mode='concate'),
         Conv2d_Block((1, 1), num_filters=num_filters, strides=1, groups=1, auto_pad=True, use_bias=False,
                      activation=None, normalization='batch')),
         Conv2d_Block((1, 1), num_filters=num_filters, strides=1, groups=1, auto_pad=True, use_bias=False,
-                     activation=None, normalization='batch'), mode=ShortcutMode.add, activation='relu')
+                     activation=None, normalization='batch'), mode='add', activation='relu')
 
 
 def conv_dw(num_filters, strides):
