@@ -20,7 +20,7 @@ from itertools import repeat
 
 
 from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy,to_tensor,Layer,Sequential,get_signature
+from ..backend.pytorch_backend import to_numpy,to_tensor,Layer,Sequential
 from ..layers.pytorch_layers import *
 from ..layers.pytorch_activations import  get_activation,Identity,Relu
 from ..layers.pytorch_normalizations import get_normalization
@@ -58,7 +58,7 @@ cfgs = {
 }
 
 
-def make_vgg_layers(cfg, num_classes=1000,input_shape=(3,224,224)):
+def make_vgg_layers(cfg, num_classes=1000,input_shape=(3,224,224),include_top=True):
     layers = []
     in_channels = 3
     block=1
@@ -78,13 +78,14 @@ def make_vgg_layers(cfg, num_classes=1000,input_shape=(3,224,224)):
             vgg.add_module('block{0}_relu{1}'.format(block, conv),Relu())
             conv+=1
             in_channels = v
-    vgg.add_module('avgpool',AdaptiveAvgPool2d((7,7)))
-    vgg.add_module('flattened', Flatten())
-    vgg.add_module('fc1',Dense(4096,use_bias=True, activation='relu'))
-    vgg.add_module('drop1', Dropout(0.5))
-    vgg.add_module('fc2', Dense(4096, use_bias=True,activation='relu'))
-    vgg.add_module('drop2', Dropout(0.5))
-    vgg.add_module('fc3', Dense(num_classes,use_bias=True,activation='sigmoid'))
+    if include_top==True:
+        vgg.add_module('avgpool',AdaptiveAvgPool2d((7,7)))
+        vgg.add_module('flattened', Flatten())
+        vgg.add_module('fc1',Dense(4096,use_bias=True, activation='relu'))
+        vgg.add_module('drop1', Dropout(0.5))
+        vgg.add_module('fc2', Dense(4096, use_bias=True,activation='relu'))
+        vgg.add_module('drop2', Dropout(0.5))
+        vgg.add_module('fc3', Dense(num_classes,use_bias=True,activation='sigmoid'))
 
 
     model = ImageClassificationModel(input_shape=input_shape, output=vgg)
