@@ -17,7 +17,7 @@ from functools import partial
 import numpy as np
 from ..backend.common import *
 from ..backend.cntk_backend import *
-from .trainers import OptimizerMixin
+from .trainers import OptimizerBase
 
 
 
@@ -685,7 +685,7 @@ class Optimizer(object):
 
 
 
-class RAdam(Optimizer,OptimizerMixin):
+class RAdam(Optimizer, OptimizerBase):
     def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, degenerated_to_sgd=True):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
