@@ -9,16 +9,17 @@ from math import *
 from torch.nn.modules.loss import _Loss, _WeightedLoss
 from torch.nn import init
 from torch.nn import _reduction as _Reduction
+from ..data.image_common import *
 from ..backend.common import *
 from ..layers.pytorch_activations import sigmoid
 from ..backend.pytorch_backend import *
 from ..backend.pytorch_ops import *
-
-
+from torchvision import transforms
+from torchvision.transforms import functional as tvf
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
 _session = get_session()
 
-__all__ = ['MSELoss', 'CrossEntropyLoss', 'NullLoss', 'BCELoss', 'F1ScoreLoss', 'L1Loss', 'SmoothL1Loss', 'L2Loss',
+__all__ = ['MSELoss', 'CrossEntropyLoss', 'NullLoss', 'BCELoss', 'F1ScoreLoss', 'L1Loss', 'SmoothL1Loss', 'L2Loss','CosineSimilarityLoss',
            'ExponentialLoss','ItakuraSaitoLoss', 'make_onehot', 'MS_SSIMLoss', 'CrossEntropyLabelSmooth', 'mixup_criterion', 'DiceLoss',
            'IouLoss', 'focal_loss_with_logits', 'FocalLoss', 'SoftIoULoss', 'CenterLoss', 'TripletLoss',
            'LovaszSoftmax', 'PerceptionLoss', 'EdgeLoss', 'TransformInvariantLoss', 'get_loss']
@@ -185,6 +186,14 @@ class ItakuraSaitoLoss(_Loss):
         else:
             raise ValueError('output and target shape should the same in ItakuraSaitoLoss. ')
 
+class CosineSimilarityLoss(_Loss):
+    def __init__(self, ):
+        super(CosineSimilarityLoss, self).__init__()
+
+    def forward(self, output, target):
+        return 1 - torch.cosine_similarity(output, target)
+
+
 
 def make_onehot(labels, classes):
     one_hot_shape = list(labels.size())
@@ -202,11 +211,11 @@ def gaussian(window_size, sigma=1.5):
 def create_window(window_size, channel):
     _1D_window = gaussian(window_size, 1.5).unsqueeze(1)
     _2D_window = _1D_window.mm(_1D_window.t()).float().unsqueeze(0).unsqueeze(0)
-    window = Variable(_2D_window.expand(channel, 1, window_size, window_size).contiguous())
+    window = _2D_window.expand(channel, 1, window_size, window_size).contiguous()
     return window
 
 
-def ssim(img1, img2, window_size=11, window=None, size_average=True, full=False, val_range=None):
+def ssim(img1, img2, window_size=11, window=None,  full=False, val_range=None):
     # Value range can be different from 255. Other common ranges are 1 (sigmoid) and 2 (tanh).
     if val_range is None:
         if torch.max(img1) > 128:
@@ -248,24 +257,23 @@ def ssim(img1, img2, window_size=11, window=None, size_average=True, full=False,
 
     ssim_map = ((2 * mu1_mu2 + C1) * v1) / ((mu1_sq + mu2_sq + C1) * v2)
 
-    if size_average:
-        ret = ssim_map.mean()
-    else:
-        ret = ssim_map.mean(1).mean(1).mean(1)
+
+    ret = ssim_map.mean()
+
 
     if full:
         return ret, cs
     return ret
 
 
-def msssim(img1, img2, window_size=11, size_average=True, val_range=None, normalize=False):
+def msssim(img1, img2, window_size=11,  val_range=None, normalize=False):
     device = img1.device
     weights = torch.FloatTensor([0.0448, 0.2856, 0.3001, 0.2363, 0.1333]).to(device)
     levels = weights.size()[0]
     mssim = []
     mcs = []
     for _ in range(levels):
-        sim, cs = ssim(img1, img2, window_size=window_size, size_average=size_average, full=True, val_range=val_range)
+        sim, cs = ssim(img1, img2, window_size=window_size,  full=True, val_range=val_range)
         mssim.append(sim)
         mcs.append(cs)
 
@@ -292,11 +300,10 @@ class MS_SSIMLoss(_Loss):
         super(MS_SSIMLoss, self).__init__()
         self.reduction = reduction
         self.window_size = window_size
-        self.reduction = reduction
         self.channel = 3
 
     def forward(self, output, target):
-        return 1-msssim(output, target, window_size=self.window_size, size_average=True, normalize=True)
+        return 1-msssim(output, target, window_size=self.window_size,normalize=True)
 
 
 class CrossEntropyLabelSmooth(_Loss):
@@ -734,38 +741,42 @@ class StyleLoss(nn.Module):
 
 
 class PerceptionLoss(_Loss):
-    def __init__(self, vgg16, reduction="mean", reduced=False):
+    def __init__(self, net, reduction="mean"):
         super(PerceptionLoss, self).__init__()
-        self.vgg_model = vgg16
-        self.vgg_model.trainable = False
-        self.vgg_model.eval()
-
+        self.ref_model = net
+        self.ref_model.trainable = False
+        self.ref_model.eval()
         self.layer_name_mapping = {'3': "block1_conv2", '8': "block2_conv2", '15': "block3_conv3", '22': "block4_conv3"}
-        for name, module in self.vgg_model.named_modules():
+        for name, module in self.ref_model.named_modules():
             if name in list(self.layer_name_mapping.values()):
                 module.keep_output = True
 
         self.reduction = reduction
-        self.reduced = reduced
-        self.styleloss = StyleLoss()
         self.to(_device)
 
+    def preprocess(self,img):
+        return ((img+1)/2)*to_tensor([[0.485, 0.456, 0.406]]).unsqueeze(-1).unsqueeze(-1)+to_tensor([[0.229, 0.224, 0.225]]).unsqueeze(-1).unsqueeze(-1)
+
+
+
     def forward(self, output, target):
         target_features = OrderedDict()
         output_features = OrderedDict()
-        _ = self.vgg_model(output)
+        _ = self.ref_model(self.preprocess(output))
         for item in self.layer_name_mapping.values():
-            output_features[item] = getattr(self.vgg_model, item).output
+            output_features[item] = getattr(self.ref_model, item).output
 
-        _ = self.vgg_model(target)
+        _ = self.ref_model(self.preprocess(target))
         for item in self.layer_name_mapping.values():
-            target_features[item] = getattr(self.vgg_model, item).output.detach()
+            target_features[item] = getattr(self.ref_model, item).output.detach()
 
         loss = 0
+        num_filters=0
         for i in range(len(self.layer_name_mapping)):
             b,c,h,w=output_features.value_list[i].shape
-            loss += ((output_features.value_list[i] - target_features.value_list[i]) ** 2).mean()/(h*w)
-        return loss
+            loss += ((output_features.value_list[i] - target_features.value_list[i]) ** 2).sum()/(h*w)
+            num_filters+=c
+        return loss/(output.size(0)*num_filters)
 
 
 class EdgeLoss(_Loss):
