@@ -72,12 +72,9 @@ def psnr(output, target):
     if input_tensor.shape != target_tensor.shape :
         raise ValueError(
             'input shape {0} is not competable with target shape {1}'.format(input_tensor.shape, target_tensor.shape))
-    diff = input_tensor - target_tensor
-    diff=diff ** 2.
-    diff = diff.view(diff.size(0),-1).mean()
-    return  10 * torch.log(255**2 / diff)/math.log(10)
-
-
+    rmse = ((input_tensor - target_tensor) ** 2).mean().sqrt()
+    psnr = 20 * (1 / rmse).log10_()
+    return psnr
 
 
 def mean_absolute_error(output, target):
@@ -87,7 +84,7 @@ def mean_absolute_error(output, target):
     if input_tensor.shape != target_tensor.shape:
         raise ValueError(
             'input shape {0} is not competable with target shape {1}'.format(input_tensor.shape, target_tensor.shape))
-    return torch.abs(input_tensor, target_tensor).mean()
+    return torch.abs(input_tensor- target_tensor).mean()
 mae=mean_absolute_error
 
 
