@@ -18,34 +18,34 @@ from functools import partial
 import numpy as np
 from ..backend.common import get_session,addindent,get_time_suffix,get_class,format_time,get_terminal_size,snake2camel,camel2snake
 from ..backend.pytorch_backend import *
-from .trainers import OptimizerMixin
+from .trainers import OptimizerBase
 
 
 __all__ = ['Adam','SGD','LBFGS','Adadelta','Adagrad','RMSprop','RAdam','PlainRAdam','AdamW','Lookahead','Ranger','get_optimizer']
 
-class Adam(optim.Adam,OptimizerMixin):
+class Adam(optim.Adam, OptimizerBase):
     pass
 
 
 
-class SGD(optim.SGD,OptimizerMixin):
+class SGD(optim.SGD, OptimizerBase):
     pass
 
-class LBFGS(get_class('LBFGS',['torch.optim']),OptimizerMixin):
+class LBFGS(get_class('LBFGS',['torch.optim']), OptimizerBase):
     pass
 
-class Adadelta(get_class('Adadelta',['torch.optim']),OptimizerMixin):
+class Adadelta(get_class('Adadelta',['torch.optim']), OptimizerBase):
     pass
 
-class Adagrad(get_class('Adagrad',['torch.optim']),OptimizerMixin):
+class Adagrad(get_class('Adagrad',['torch.optim']), OptimizerBase):
     pass
 
-class RMSprop(get_class('RMSprop',['torch.optim']),OptimizerMixin):
+class RMSprop(get_class('RMSprop',['torch.optim']), OptimizerBase):
     pass
 
 
 
-class RAdam(Optimizer,OptimizerMixin):
+class RAdam(Optimizer, OptimizerBase):
     def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, degenerated_to_sgd=True):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
@@ -133,7 +133,7 @@ class RAdam(Optimizer,OptimizerMixin):
 
         return loss
 
-class PlainRAdam(Optimizer,OptimizerMixin):
+class PlainRAdam(Optimizer, OptimizerBase):
     def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, degenerated_to_sgd=True):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
@@ -210,7 +210,7 @@ class PlainRAdam(Optimizer,OptimizerMixin):
         return loss
 
 
-class AdamW(Optimizer,OptimizerMixin):
+class AdamW(Optimizer, OptimizerBase):
     def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, warmup=0):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
@@ -282,7 +282,7 @@ class AdamW(Optimizer,OptimizerMixin):
         return loss
 
 
-class Lookahead(Optimizer,OptimizerMixin):
+class Lookahead(Optimizer, OptimizerBase):
     def __init__(self, optimizer, k=5, alpha=0.5):
         self.optimizer = optimizer
         self.k = k
@@ -335,7 +335,7 @@ class Lookahead(Optimizer,OptimizerMixin):
         param_group["counter"] = 0
         self.optimizer.add_param_group(param_group)
 
-class Ranger(Optimizer,OptimizerMixin):
+class Ranger(Optimizer, OptimizerBase):
     '''
     https://github.com/lessw2020/Ranger-Deep-Learning-Optimizer/blob/master/ranger/ranger.py
     '''
