@@ -1,12 +1,9 @@
 import os
 import copy
-from ..misc.ipython_utils import is_in_ipython
-
-if is_in_ipython():
-    from IPython import display
 import inspect
 import time
 import random
+import sys
 import string
 import uuid
 import torch
@@ -16,6 +13,7 @@ from functools import partial
 import numpy as np
 from ..backend.common import *
 from ..backend.pytorch_backend import *
+from ..backend.pytorch_ops import *
 from .pytorch_optimizers import get_optimizer
 from .pytorch_losses import get_loss
 from .pytorch_metrics import get_metric
@@ -27,7 +25,7 @@ from .trainers import ModelBase, OptimizerBase, progress_bar
 from ..data.image_common import *
 
 __all__ = ['TrainingItem', 'Model', 'ImageClassificationModel', 'ImageDetectionModel', 'ImageGenerationModel',
-           'ImageSegmentationModel']
+           'ImageSegmentationModel','FaceRecognitionModel']
 
 _, term_width = get_terminal_size()
 term_width = int(term_width)
@@ -101,7 +99,8 @@ class Model(ModelBase):
         if isinstance(output, (Layer, nn.Module)):
             output.input_shape = input_shape
             # output.cpu()
-            dummay_input = to_tensor(np.random.standard_normal((1, *input_shape)).astype(np.float32))
+            output.to(output.device)
+            dummay_input = to_tensor(np.random.standard_normal((2,)+tuple(input_shape)).astype(np.float32)).to(output.device)
             out = output(dummay_input)
             self._model = output
 
@@ -119,7 +118,7 @@ class Model(ModelBase):
             for op in output:
                 if isinstance(op, (Layer, nn.Module)):
                     output_list.append(op)
-            dummay_input = to_tensor(np.random.standard_normal((2, *input_shape)).astype(np.float32))
+            dummay_input = to_tensor(np.random.standard_normal((2,)+tuple(input_shape)).astype(np.float32)).to(output.device)
             model = Combine(output_list)
             outs = model(dummay_input)
             self._model = model
@@ -137,20 +136,14 @@ class Model(ModelBase):
             self.signature['x']=to_list(self._model.size())[1:]
         else:
             raise ValueError('Invalid output')
-        if not isinstance(output, (np.ndarray, torch.Tensor)):
-            for name, module in self._model.named_modules():
-                try:
-                    if hasattr(module, '_name'):
-                        if name.isdigit():
-                            module._name = name + '.' + module.name
-                        else:
-                            module._name = name
-                except:
-                    PrintException()
 
-            self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
-            self._model.to(self.device)
-            self.training_context['current_model'] = self._model
+
+
+        self.training_context['current_model'] = self._model
+        self.device = self._model.device
+        save_path = os.path.join('Models', '{0}.pth.tar_'.format(self._model.name))
+        self.save_path =sanitize_path(make_dir_if_need(save_path))
+        self.training_context['save_path'] = self.save_path
 
     @property
     def outputs(self):
@@ -164,7 +157,7 @@ class Model(ModelBase):
 
             return self._outputs
         elif len(self._outputs) > 1:
-            dummay_input = to_tensor(np.random.standard_normal((1, *self._model.input_shape)).astype(np.float32))
+            dummay_input = to_tensor(np.random.standard_normal((2,) + tuple(self.inputs.value_list[0])).astype(np.float32)).to(self._model.device)
             outs = self._model(dummay_input)
             if len(self._outputs) == len(outs):
                 for i in range(len(outs)):
@@ -313,7 +306,9 @@ class Model(ModelBase):
         print('{0} signature:{1}'.format(alias, self._losses[alias].signature.item_list))
         return self
 
-    def with_metric(self, metric, output_idx=0, name='', **kwargs):
+    def with_metric(self, metric, output_idx=0,collect_history=None ,name='', **kwargs):
+        if collect_history is None:
+            collect_history=True
         alias = name
         argnames = OrderedDict()
         if (alias is None or len(alias) == 0) and hasattr(metric, '__name__'):
@@ -389,6 +384,7 @@ class Model(ModelBase):
                         argnames[target] = targets[target]
         self._metrics[alias].__name__ = alias
         self._metrics[alias].signature = argnames
+        self._metrics[alias].collect_history=collect_history
         print('{0} signature:{1}'.format(alias, self._metrics[alias].signature.item_list))
         return self
 
@@ -428,6 +424,8 @@ class Model(ModelBase):
         return self
 
     def with_model_save_path(self, save_path, **kwargs):
+        if save_path is None or len(save_path)==0:
+            save_path=os.path.join('Models','{0}.pth.tar_'.format(self.name))
         self.save_path = make_dir_if_need(save_path)
         self.training_context['save_path'] = self.save_path
         return self
@@ -462,8 +460,7 @@ class Model(ModelBase):
                 self.inputs[self.inputs.key_list[0]] = input_shape_list
 
             dummay_input = to_tensor(np.random.standard_normal((2, *input_shape_list))).to(
-                torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu")).to(
-                self._model.weights[0].data.dtype)
+                torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu"))
             out = self._model(dummay_input)
             if isinstance(out, torch.Tensor) == len(self.targets) == 1:
                 self.targets[self.targets.key_list[0]] = list(out.size())[1:] if len(out.size()) > 1 else []
@@ -625,15 +622,30 @@ class Model(ModelBase):
     def get_current_loss(self):
         return self.training_context['current_loss']
 
-    def do_gradient_update(self, log_gradients=False, keep_gradients_history=False):
+    def do_gradient_update(self, log_gradients=False):
+        if self.training_context['stop_update'] <1:
+            self.training_context['current_loss'].backward(retain_graph=self.training_context['retain_graph'])
+            #only check once every epoch start.
+            if self.training_context['current_batch']==0:
+                if isinstance(self._model,nn.Module):
+                    for name,para in self._model.named_parameters():
+                        try:
+                            if para is not None  and para.grad is not None:
+                                grad_norm=para.grad.norm()
+                                if not 0<grad_norm<1e5:
+                                    sys.stderr.write('warning...Gradient norm exceed 1e5 nor less-or-equal zero')
+                        except:
+                            PrintException()
+                elif isinstance(self._model,torch.Tensor):
+                    grad_norm = self._model.grad.norm()
+                    if not 0 < grad_norm < 1e5:
+                        sys.stderr.write('Gradient norm exceed 1e5 nor less-or-equal zero')
 
-        self.training_context['current_loss'].backward(retain_graph=self.training_context['retain_graph'])
+            for callback in self.training_context['callbacks']:
+                callback.on_optimization_step_start(self.training_context)
 
-        for callback in self.training_context['callbacks']:
-            callback.on_optimization_step_start(self.training_context)
-
-        if keep_gradients_history:
-            self.log_gradient(keep_gradients_history)
+            if log_gradients:
+                self.log_gradient()
 
         if self.training_context['stop_update'] == 0:
             self.optimizer.step(self.get_current_loss)
@@ -652,43 +664,42 @@ class Model(ModelBase):
                 self.lr_scheduler.step(np.array(self.training_context['metrics'][list(self._metrics.keys())[0]]).mean())
                 self.training_context['current_lr'] = self.optimizer.lr
 
-    def log_gradient(self, grads=None, keep_gradients_history=False):
-        grad_dict = {}
+    def log_gradient(self, grads=None):
+        grad_dict =OrderedDict()
         if isinstance(self._model, nn.Module):
             for i, (k, v) in enumerate(self._model.named_parameters()):
                 grad_dict[k] = to_numpy(v.grad)
+            self.gradients_history.append(grad_dict)
 
-            if keep_gradients_history:
-                self.gradients_history.append(grad_dict)
-
-    def log_weight(self, weghts=None, keep_weghts_history=False):
-        weight_dict = {}
+    def log_weight(self, weghts=None):
+        weight_dict =OrderedDict()
         if isinstance(self._model, nn.Module):
             for k, v in self._model.named_parameters():
                 weight_dict[k] = to_numpy(v.data)
-            if keep_weghts_history:
-                self.weights_history.append(weight_dict)
+
+            self.weights_history.append(weight_dict)
 
     def save_model(self, save_path=None):
         for callback in self.training_context['callbacks']:
             callback.on_model_saving_start(self.training_context)
 
-        if save_path is None or len(save_path) == 0:
-            save_path = self.training_context.get('save_path', self.save_path)
+        if is_nan(self._model):
+            raise ValueError(self._get_name() + '  nan detected!!')
 
-        if isinstance(self._model, nn.Module):
+        if isinstance(self._model,nn.Module):
+            save_path=self.get_save_path(save_path,default_folder='Models',default_file_name= '{0}_epoch{1}.pth.tar_'.format(self._model.name,self.training_context['current_epoch']))
             self._model.eval()
-            if save_path is None or len(save_path) == 0:
-                save_path = 'Models/model_{0}_epoch{1}.pth'.format(self._model.name,self.training_context['current_epoch'])
-            torch.save({'state_dict': self._model.state_dict()}, save_path + '.tar_')
-            # save safety
-            shutil.copyfile(save_path + '.tar_', save_path + '.tar')
+
+            torch.save({'state_dict': self._model.state_dict()}, save_path)
             self._model.train()
+            shutil.copy(save_path, save_path.replace('.pth.tar_','.pth.tar'))
+
         elif isinstance(self._model,torch.Tensor):
-            if save_path is None or len(save_path) == 0:
-                save_path = 'Models/model_{0}_epoch{1}.npy'.format(self.name,self.training_context['current_epoch'])
+            save_path = self.get_save_path(save_path, default_folder='Models',default_file_name='{0}_epoch{1}.npy_'.format(self._model.name, self.training_context[ 'current_epoch']))
+
             numpy_model=to_numpy(self._model)
-            np.save(save_path,numpy_model)
+            np.save( sanitize_path(save_path),numpy_model)
+            shutil.copy(save_path, save_path.replace('.npy_', '.npy'))
             os.strerror('Yor model is a Tensor not a nn.Module, it has saved as numpy array(*.npy) successfully. ')
         else:
             raise ValueError('only Layer or nn.Module as model can export to onnx, yours model is {0}'.format(type(self._model)))
@@ -696,6 +707,8 @@ class Model(ModelBase):
 
     def save_onnx(self, file_path, dynamic_axes=None):
         if isinstance(self._model,nn.Module):
+            save_path = self.get_save_path(file_path, default_folder='Models',default_file_name='{0}_epoch{1}.onnx_'.format(self._model.name, self.training_context[ 'current_epoch']))
+
             import torch.onnx
             self._model.eval()
             dummy_input = torch.randn(1, *self._model.input_shape.tolist(),
@@ -716,7 +729,7 @@ class Model(ModelBase):
                     dynamic_axes[out] = {0: 'batch_size'}
             torch.onnx.export(self._model,  # model being run
                               dummy_input,  # model input (or a tuple for multiple inputs)
-                              file_path,  # where to save the model (can be a file or file-like object)
+                              save_path,  # where to save the model (can be a file or file-like object)
                               export_params=True,  # store the trained parameter weights inside the model file
                               opset_version=11,  # the ONNX version to export the model to
                               do_constant_folding=True,  # whether to execute constant folding for optimization
@@ -724,6 +737,7 @@ class Model(ModelBase):
                               output_names=output_names,  # the model's output names
                               dynamic_axes=dynamic_axes)
             self._model.train()
+            shutil.copy(save_path, save_path.replace('.onnx_', '.onnx'))
         else:
             raise ValueError('only Layer or nn.Module as model can export to onnx, yours model is {0}'.format(type(self._model)))
 
@@ -814,8 +828,6 @@ class ImageClassificationModel(Model):
         super(ImageClassificationModel, self).__init__(inputs, output, input_shape)
 
         self._class_names = []
-        self.preprocess_flow = []
-
         self._idx2lab = {}
         self._lab2idx = {}
 
@@ -830,16 +842,7 @@ class ImageClassificationModel(Model):
             self._lab2idx = {v: k for k, v in enumerate(self._class_names)}
             self._idx2lab = {k: v for k, v in enumerate(self._class_names)}
 
-    @property
-    def reverse_preprocess_flow(self):
-        return_list = []
-        return_list.append(reverse_image_backend_adaptive)
-        for i in range(len(self.preprocess_flow)):
-            fn = self.preprocess_flow[-1 - i]
-            if fn.__qualname__ == 'normalize.<locals>.img_op':
-                return_list.append(unnormalize(fn.mean, fn.std))
-        return_list.append(array2image)
-        return return_list
+
 
     def index2label(self, idx: int):
         if self._idx2lab is None or len(self._idx2lab.items()) == 0:
@@ -973,9 +976,68 @@ class ImageGenerationModel(Model):
 
             for func in self.reverse_preprocess_flow:
                 if inspect.isfunction(func):
+                    result = func(result)
+            result = array2image(result)
+            return result
+
+
+class FaceRecognitionModel(Model):
+    def __init__(self, inputs=None, output=None, input_shape=None):
+        super(FaceRecognitionModel, self).__init__(inputs, output, input_shape)
+
+        self._class_names = []
+        self.preprocess_flow = []
+
+        self._idx2lab = {}
+        self._lab2idx = {}
+
+
+
+    @property
+    def reverse_preprocess_flow(self):
+        return_list = []
+        return_list.append(reverse_image_backend_adaptive)
+        for i in range(len(self.preprocess_flow)):
+            fn = self.preprocess_flow[-1 - i]
+            if fn.__qualname__ == 'normalize.<locals>.img_op':
+                return_list.append(unnormalize(fn.mean, fn.std))
+        return_list.append(array2image)
+        return return_list
+
+    def get_embedded(self,img_path):
+        def norm(x):
+            b = np.sqrt(np.sum(np.square(x)))
+            return x / (b if b != 0 else 1)
+
+        img = image2array(img_path)
+        img = resize((224, 224), keep_aspect=True)(img)
+        img = normalize([131.0912, 103.8827, 91.4953], [1, 1, 1])(img)
+        img = to_tensor(np.expand_dims(img.transpose([2, 0, 1]), 0))
+        embedding = self.model(img)[0]
+        return norm(embedding)
+
+    def infer_single_image(self, img):
+        def norm(x):
+            b = np.sqrt(np.sum(np.square(x)))
+            return x / (b if b != 0 else 1)
+        if isinstance(self._model,Layer) and self._model.built:
+            self._model.eval()
+            img = image2array(img)
+            if img.shape[-1] == 4:
+                img = img[:, :, :3]
+
+            for func in self.preprocess_flow:
+                if inspect.isfunction(func) and func is not image_backend_adaptive:
                     img = func(img)
-            img = array2image(img)
-            return img
+            img = image_backend_adaptive(img)
+            inp = to_tensor(np.expand_dims(img, 0)).to(torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu")).to(self._model.weights[0].data.dtype)
+            result = self._model(inp)[0]
+            embedding = to_numpy(result)
+            return norm(embedding)
+
+        else:
+            raise ValueError('the model is not built yet.')
+
 
 
 class LanguageModel(Model):
