@@ -60,8 +60,9 @@ class Model(ModelBase):
             if input_shape is None:
                 pass
             else:
-                input_shape = to_list(input_shape)
+                input_shape = tuple(to_list(input_shape))
                 input_name = 'input_{0}'.format(len(self.inputs) + 1)
+                input_var = tf.keras.Input(input_shape)
                 # input_var = Input(input_shape, name=input_name)
                 self.inputs[input_name] = input_shape
         elif inputs is tf.keras.layers.InputLayer:
@@ -79,6 +80,7 @@ class Model(ModelBase):
                     self.inputs[k] = tf.keras.backend.int_shape(v.input_shape)
 
         if isinstance(out_var, Sequential):
+            model = tf.keras.Model(input_var, out_var)
             dummay_input = np.random.standard_normal((2, *input_shape)).astype(np.float32)
             out = out_var(dummay_input)
             out_var._set_inputs(dummay_input)
