@@ -1,28 +1,15 @@
 import os
 import sys
-import tkinter
-
-
-import matplotlib
-import platform
-
-# if platform.system() not in ['Linux', 'Darwin'] and not platform.system().startswith('CYGWIN'):
-#     matplotlib.use('TKAgg')
-import matplotlib.pyplot as plt
-from ..misc.ipython_utils import is_in_ipython
-
-if is_in_ipython():
-    from IPython import display
 import inspect
-import time
 import uuid
 import time
-import platform
 import copy
+import json
+import shutil
 from functools import partial
 import numpy as np
 from ..backend.common import to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session, \
-    snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict
+    snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict,split_path,sanitize_path
 from ..data.data_provider import *
 from ..callbacks import *
 from ..callbacks.visualization_callbacks import *
@@ -37,13 +24,16 @@ if _backend == 'pytorch':
     import torch
     import torch.nn as nn
     from ..backend.pytorch_backend import *
+    from ..backend.pytorch_ops import *
 elif _backend == 'tensorflow':
     import tensorflow as tf
     from ..backend.tensorflow_backend import *
+    from ..backend.tensorflow_ops import *
 elif _backend == 'cntk':
     import cntk as C
     from cntk.learners import *
     from ..backend.cntk_backend import *
+    from ..backend.cntk_ops import *
 
 _, term_width = get_terminal_size()
 term_width = int(term_width)
@@ -198,6 +188,7 @@ class ModelBase(object):
         self.base_lr = None
         self.warmup = 0
         self.sample_collect_history = []
+        self.preprocess_flow = []
 
         self.epoch_loss_history = OrderedDict()
         self.epoch_loss_history['total_losses'] = []
@@ -380,6 +371,43 @@ class ModelBase(object):
             #self.signature = namedtuple('signature', arg_names)
             print(self.signature.key_list)
 
+    @property
+    def reverse_preprocess_flow(self):
+        return_list = []
+        return_list.append(reverse_image_backend_adaptive)
+        for i in range(len(self.preprocess_flow)):
+            fn = self.preprocess_flow[-1 - i]
+            if fn.__qualname__ == 'normalize.<locals>.img_op':
+                return_list.append(unnormalize(fn.mean, fn.std))
+        return_list.append(array2image)
+        return return_list
+
+    def data_preprocess(self, img_data):
+        if img_data.ndim==4:
+            return to_tensor(to_numpy([self.data_preprocess(im) for im in img_data]))
+        if len(self.preprocess_flow) == 0:
+            return image_backend_adaptive(img_data)
+        if isinstance(img_data, np.ndarray):
+            for fc in self.preprocess_flow:
+                if not fc.__qualname__.startswith('random_') or  'crop' in fc.__qualname__  or  'rescale' in fc.__qualname__  or  (fc.__qualname__.startswith('random_') and random.randint(0,10)%2==0):
+                    img_data = fc(img_data)
+            img_data = to_tensor(image_backend_adaptive(img_data)).unsqueeze(0)
+            return img_data
+        else:
+            return img_data
+
+    def reverse_data_preprocess(self, img_data: np.ndarray):
+        if img_data.ndim==4:
+            return to_numpy([self.reverse_data_preprocess(im) for im in img_data])
+        if len(self.reverse_preprocess_flow) == 0:
+            return reverse_image_backend_adaptive(img_data)
+        if isinstance(img_data, np.ndarray):
+            # if img_data.ndim>=2:
+            for fc in self.reverse_preprocess_flow:
+                img_data = fc(img_data)
+            img_data = reverse_image_backend_adaptive(img_data)
+        return img_data
+
     def _initial_graph(self, inputs=None, output=None, input_shape=None):
         pass
 
@@ -495,7 +523,7 @@ class ModelBase(object):
         # zero grad
         pass
 
-    def do_gradient_update(self, log_gradients=False, keep_gradients_history=False):
+    def do_gradient_update(self, log_gradients=False):
         pass
 
     def do_post_gradient_update(self):
@@ -521,21 +549,53 @@ class ModelBase(object):
         # zero grad
         pass
 
-    def log_gradient(self, grads=None, keep_gradients_history=False):
+    def log_gradient(self, grads=None):
         raise NotImplementedError
 
-    def log_weight(self, weghts=None, keep_weghts_history=False):
+    def log_weight(self, weghts=None):
         raise NotImplementedError
 
     def merge_grads(self, old_grads, new_grades):
         raise NotImplementedError
 
+    def get_save_path(self,save_path='',default_folder='log',default_file_name=''):
+        _,default_filename,default_ext=split_path(default_file_name)
+        if save_path is None or len(save_path) == 0:
+            save_path = self.training_context.get('save_path', getattr(self, 'save_path',''))
+
+        folder,filename,ext=split_path(save_path)
+        if folder=='':
+            folder=default_folder
+        if filename=='':
+            filename=default_filename
+        save_path = os.path.join(folder, filename+default_ext)
+        make_dir_if_need(sanitize_path(save_path))
+        return sanitize_path(save_path)
+
+
+
     def save_model(self, file_path, ):
         raise NotImplementedError
 
     def save_onnx(self, file_path):
         raise NotImplementedError
 
+    def save_history(self, file_path=None):
+        default_file_name = '{0}_history_{1}.json_'.format(self._model.name, self.training_context['execution_id'])
+        save_path = self.get_save_path(file_path, default_folder='Log',default_file_name=default_file_name)
+        folder,filename,ext=split_path(save_path)
+        save_path=os.path.join(folder,default_file_name)
+        out=OrderedDict()
+        out['batch_loss_history']=self.batch_loss_history
+        out['batch_metric_history'] = self.batch_metric_history
+        out['epoch_loss_history'] = self.epoch_loss_history
+        out['epoch_metric_history'] = self.epoch_metric_history
+        with open(save_path, 'w') as f:
+            jstring=json.dumps(out, indent=4)
+            f.write(jstring)
+            shutil.copy(save_path, save_path.replace('.json_', '.json'))
+
+
     def save_weights(self, file_path):
         raise NotImplementedError
 
@@ -544,13 +604,29 @@ class ModelBase(object):
 
     def print_batch_progress(self, print_batch_progress_frequency):
         slice_cnt = sum(self.sample_collect_history[-1 * print_batch_progress_frequency:])
+        metric_strings=[]
+        for k, v in self._metrics.items():
+            format_string='.3%'
+            collect_history = self._metrics[k].collect_history
+            if collect_history!=False and k in self.batch_metric_history and len(self.batch_metric_history[k])>=slice_cnt:
+                metric_value=np.array(self.batch_metric_history[k][-1 * slice_cnt:]).mean()
+                if metric_value.item()>5:
+                    format_string = '.3f'
+                elif metric_value.item()<0.005:
+                    format_string = '.3e'
+                metric_strings.append('{0}: {1:<8{2}}'.format(k,metric_value,format_string))
+            elif collect_history==False and k in self.training_context['tmp_metrics']:
+                metric_value = np.array(self.training_context['tmp_metrics'][k]).mean()
+                if metric_value.item() > 5:
+                    format_string = '.3%'
+                metric_strings.append('{0}: {1:<8{2}}'.format(k,metric_value,format_string))
+                self.training_context['tmp_metrics'][k]=[]
+
 
         progress_bar(self.training_context['current_batch'], self.training_context['total_batch'],
-                     'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.3e}| epoch: {3}'.format(
-                         np.array(self.batch_loss_history['total_losses'][-1 * slice_cnt:]).mean(), ','.join(
-                             ['{0}: {1:<8.3%}'.format(k, np.array(v[-1 * slice_cnt:]).mean()) for k, v in
-                              self.batch_metric_history.items()]), self.training_context['current_lr'],
-                         self.training_context['current_epoch']), name=self.name)
+                 'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.3e}| epoch: {3}'.format(
+                     np.array(self.batch_loss_history['total_losses'][-1 * slice_cnt:]).mean(), ','.join(metric_strings), self.training_context['current_lr'],
+                     self.training_context['current_epoch']), name=self.name)
 
     def print_epoch_progress(self, print_epoch_progress_frequency):
         progress_bar(self.training_context['current_epoch'], self.training_context['total_epoch'],
@@ -636,18 +712,25 @@ class ModelBase(object):
                         loss_weight = 1
                         if k in self.loss_weights:
                             loss_weight = self.loss_weights[k]
-                        this_loss = to_tensor(loss_weight)*try_map_args_and_call(v, train_data, self.training_context['data_feed'])  # v.forward(output, target) if hasattr(v, 'forward') else v(
+
+                        this_loss = to_tensor(loss_weight)*try_map_args_and_call(v, train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else torch.tensor(0)# v.forward(output, target) if hasattr(v, 'forward') else v(
                         # output, target)
 
                         if isinstance(this_loss, tuple):
-                            overall_loss = this_loss[0]
-                            for i in range(1, len(this_loss)):
-                                overall_loss += this_loss[i]
+                            overall_loss =0
+                            for i in range(len(this_loss)):
+                                if is_abnormal_number(this_loss[i]):
+                                    sys.stderr.write('Loss {0} have abnormal number (nan, inf,-inf), trident will skip it automaticly, please check anything wrong!!!/n'.format(k))
+                                else:
+                                    overall_loss += this_loss[i]
                             self.training_context['current_loss'] += overall_loss
                             if is_collect_data:
                                 self.training_context['losses'][k].append(float(to_numpy(overall_loss)))
                         else:
-                            self.training_context['current_loss'] += this_loss
+                            if is_abnormal_number(this_loss):
+                                sys.stderr.write( 'Loss {0} have abnormal number (nan, inf,-inf), trident will skip it automaticly, ' 'please check anything wrong!!!/n'.format(k))
+                            else:
+                                self.training_context['current_loss'] += this_loss
                             if is_collect_data:
                                 self.training_context['losses'][k].append(float(to_numpy(this_loss)))
                     except Exception as e:
@@ -664,7 +747,7 @@ class ModelBase(object):
                     if k + '_Loss' not in self.training_context['losses']:
                         self.training_context['losses'][k + '_Loss'] = []
 
-                    this_loss = try_map_args_and_call(v, train_data, self.training_context['data_feed'])
+                    this_loss = try_map_args_and_call(v, train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else torch.tensor(0)
                     overall_loss=None
                     if isinstance(this_loss, tuple):
                         overall_loss = this_loss[0]
@@ -683,7 +766,7 @@ class ModelBase(object):
                 for k, v in self._model_regs.items():
                     if k + '_Loss' not in self.training_context['losses']:
                         self.training_context['losses'][k + '_Loss'] = []
-                    this_loss = v(self._model)
+                    this_loss = v(self._model) if  self.training_context['stop_update']<1 else torch.tensor(0)
                     overall_loss = None
                     if isinstance(this_loss, tuple):
                         overall_loss = this_loss[0]
@@ -712,12 +795,13 @@ class ModelBase(object):
 
                 # model comfirm
                 for k, v in self._constraints.items():
-                    v(self._model)
+                    if self.training_context['stop_update'] == 0 :
+                        v(self._model)
 
                 if log_weights and is_collect_data:
                     self.log_weight()
 
-                if test_data is not None and len(test_data) > 0:
+                if test_data is not None and len(test_data) > 0 and  self.training_context['stop_update']<1 :
                     output = try_map_args_and_call(self._model, test_data, self.training_context['data_feed'])
                     if isinstance(output, (list, tuple)):
                         for i in range(len(output)):
@@ -736,14 +820,17 @@ class ModelBase(object):
 
 
                 for k, v in self._metrics.items():
-                    if k not in self.training_context['metrics']:
+                    collect_history = getattr(v,'collect_history')
+                    if k not in self.training_context['metrics'] :
                         self.training_context['tmp_metrics'][k] = []
                         self.training_context['metrics'][k] = []
+                        if not collect_history==False:
+                            self.training_context['metrics'][k] = []
 
-                    this_metric = try_map_args_and_call(v, train_data, self.training_context['data_feed'])
+                    this_metric = try_map_args_and_call(v, train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else torch.tensor(0)
                     self.training_context['tmp_metrics'][k].append(to_numpy(this_metric).mean())
 
-                    if test_data is not None and len(test_data) > 0 :
+                    if test_data is not None and len(test_data) > 0 and collect_history!=False :
                         if k not in self.training_context['out_sample_metrics']:
                             self.training_context['out_sample_metrics'][k] = []
 
@@ -759,13 +846,15 @@ class ModelBase(object):
 
                 #callback's metric can keep in epoch_metric_history
                 for k, v in self.training_context['tmp_metrics'].items():
-                    if k not in self.epoch_metric_history:
-                        self.epoch_metric_history[k] = []
+                    if not getattr(self._metrics[k], 'collect_history') ==False:
+                        if k not in self.epoch_metric_history:
+                            self.epoch_metric_history[k] = []
 
                 if is_collect_data:
                     for k, v in self.training_context['tmp_metrics'].items():
-                        self.training_context['metrics'][k].append(float(to_numpy(v).mean()))
-                        self.training_context['tmp_metrics'][k] = []
+                        if not getattr(self._metrics[k], 'collect_history')== False:
+                            self.training_context['metrics'][k].append(float(to_numpy(v).mean()))
+                            self.training_context['tmp_metrics'][k] = []
 
                 # print batch progresss
                 if is_print_batch_progress:
@@ -783,7 +872,7 @@ class ModelBase(object):
                     self.training_context['print_batch_progress_frequency'] += 1
 
                 if test_data is not None and len(test_data) > 0:
-                    print('out-of-sample evaluation: ',','.join(['{0}: {1:<8.3%}'.format(k, v[-1]) for k, v in self.training_context['out_sample_metrics'].items()]))
+                    print(self.training_context['model_name']+': out-of-sample evaluation: ',','.join(['{0}: {1:<8.3%}'.format(k, v[-1]) for k, v in self.training_context['out_sample_metrics'].items()]))
 
 
                 # ON_BATCH_END
@@ -798,7 +887,8 @@ class ModelBase(object):
                 self.epoch_loss_history['total_losses'].append(
                     np.array(self.training_context['losses']['total_losses'][-1 * slice_cnt:]).mean())
                 for k, v in self.training_context['metrics'].items():
-                    self.epoch_metric_history[k].append(np.array(v[-1 * slice_cnt:]).mean())
+                    if len(v)>=slice_cnt:
+                        self.epoch_metric_history[k].append(np.array(v[-1 * slice_cnt:]).mean())
 
                 if is_print_epoch_progress:
                     self.do_on_progress_start()
@@ -855,8 +945,8 @@ class TrainingPlan(object):
         self.print_progress_frequency = 10
         self.print_progress_unit = 'batch'
         self.print_progress_on_epoch_end = False
-        self.out_sample_evaluation_frequency = 100
-        self.out_sample_evaluation_unit = 'batch'
+        self.out_sample_evaluation_frequency =1
+        self.out_sample_evaluation_unit = 'epoch'
         self.out_sample_evaluation_on_epoch_end = True
         self.save_model_frequency = -1
         self.save_model_unit = 'batch'
@@ -961,7 +1051,7 @@ class TrainingPlan(object):
         plan = cls()
         return plan
 
-    def add_training_item(self, training_item, name=''):
+    def add_training_item(self, training_item, start_epoch=0,name=''):
         n = len(self.training_items)
 
         alias=name if len(name)>0 else  training_item.name
@@ -983,6 +1073,7 @@ class TrainingPlan(object):
                     training_item.model.name ='model {0}'.format(n)
                 self.training_names[n] = 'model {0}'.format(n)
         self.training_items[n] = training_item
+        self.training_items[n].start_epoch = start_epoch
         return self
 
     def with_data_loader(self, data_loader, **kwargs):
@@ -1112,12 +1203,14 @@ class TrainingPlan(object):
 
 
 
-    def start_now(self, collect_data_inteval=1, is_resume=False):
+    def start_now(self, collect_data_inteval=1, is_resume=False,only_steps=False,max_batches=np.inf,keep_weights_history=False, keep_gradient_history=False):
         try:
-            self.execution_id = str(uuid.uuid4())[:8].__str__().replace('-', '') if self.execution_id is None else self.execution_id
+            self.execution_id = get_time_suffix()
+
             # update callback
-            if not is_resume:
+            if not is_resume or only_steps==True:
                 for item in self.training_items.values():
+                    item.training_context['execution_id']=self.execution_id
                     for callback in self.callbacks:
                         if callback not in item.callbacks:
                             #private callback
@@ -1134,20 +1227,18 @@ class TrainingPlan(object):
             data_loader = self._dataloaders.value_list[0]
             data_loader.minibatch_size = self.minibatch_size
             #generate data feed
-            self.generate_datafeed(data_loader)
 
-            if not is_resume:
-                #print(self.__repr__)
+            if not is_resume or only_steps==True:
+                self.generate_datafeed(data_loader)
                 if collect_data_inteval == 1 and len(data_loader.batch_sampler) * self.num_epochs > 1000:
                     collect_data_inteval = self.default_collect_data_inteval
+            if only_steps==True:
+                self.num_epochs=(max_batches//len(data_loader.batch_sampler))+2
 
             for epoch in range(self.num_epochs):
                 try:
-                    input = None
-                    target = None
-                    test_input = None
-                    test_target = None
                     for mbs, return_data in enumerate(data_loader):
+                        num_batches=len(data_loader.batch_sampler)*epoch+mbs
                         iter_data = OrderedDict()
                         for i in range(len(data_loader.signature.key_list)):
                             name = data_loader.signature.key_list[i]
@@ -1155,11 +1246,17 @@ class TrainingPlan(object):
 
                         #check weather need out-of-sample evaluation
                         need_out_sample_evaluation=False
-                        if self.out_sample_evaluation_on_epoch_end==True and mbs==len(data_loader.batch_sampler)-1:
+                        if  only_steps==False and self.out_sample_evaluation_on_epoch_end==True and mbs==len(data_loader.batch_sampler)-1:
                             need_out_sample_evaluation=True
-                        if self.out_sample_evaluation_unit=='batch' and mbs>0 and mbs%self.out_sample_evaluation_frequency==0:
+                        elif only_steps==True and self.out_sample_evaluation_on_epoch_end==True and num_batches==max_batches-1:
+                            need_out_sample_evaluation=True
+                        elif only_steps==False and self.out_sample_evaluation_unit=='batch' and mbs>0 and mbs%self.out_sample_evaluation_frequency==0:
                             need_out_sample_evaluation = True
-                        elif self.out_sample_evaluation_unit=='epoch' and mbs==len(data_loader.batch_sampler)-1 and epoch%self.out_sample_evaluation_frequency==0:
+                        elif only_steps==True and self.out_sample_evaluation_unit=='batch' and num_batches>0 and num_batches%self.out_sample_evaluation_frequency==0:
+                            need_out_sample_evaluation = True
+                        elif only_steps==False and self.out_sample_evaluation_unit=='epoch' and mbs==len(data_loader.batch_sampler)-1 and epoch%self.out_sample_evaluation_frequency==0:
+                            need_out_sample_evaluation = True
+                        elif only_steps==True and self.out_sample_evaluation_unit=='epoch' and num_batches==max_batches-1 :
                             need_out_sample_evaluation = True
 
 
@@ -1174,7 +1271,7 @@ class TrainingPlan(object):
 
                         # input, target = Variable(input).to(self.device), Variable(target).to(self.device)
 
-                        for k, trainitem in self.training_items.items():
+                        for trainitem_name, trainitem in zip(self.training_names.value_list,self.training_items.value_list):
                             train_data=OrderedDict()
                             test_data = None if iter_testdata is None else OrderedDict()
                             for k,v in iter_data.items():
@@ -1184,14 +1281,16 @@ class TrainingPlan(object):
                                 for k,v in iter_testdata.items():
                                     test_data[k]=v.copy()
 
-
-                            trainitem.train_model(train_data,test_data, epoch, mbs, self.num_epochs,
-                                                  len(data_loader.batch_sampler),
+                            trainitem.training_context['model_name']=trainitem_name
+                            if epoch<trainitem.start_epoch:
+                                trainitem.training_context['stop_update']=1
+                            trainitem.train_model(train_data,test_data, epoch, mbs if only_steps==False else  num_batches, self.num_epochs if only_steps==False else  1,
+                                                  len(data_loader.batch_sampler) if only_steps==False else max_batches,
                                                   is_collect_data=mbs % collect_data_inteval == 0,
                                                   is_print_batch_progress=self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0,
                                                   is_print_epoch_progress=self.print_progress_unit == 'epoch' and (
                                                               epoch + 1) % self.print_progress_frequency == 0,
-                                                  log_gradients=False, log_weights=False, accumulate_grads=False)
+                                                  log_gradients=keep_gradient_history, log_weights=keep_weights_history, accumulate_grads=False)
 
                         for k, trainitem in self.training_items.items():
                             for callback in trainitem.training_context['callbacks']:
@@ -1205,15 +1304,15 @@ class TrainingPlan(object):
                             for k, trainitem in self.training_items.items():
                                 trainitem.save_model()
 
-                        if (mbs + 1) % len(data_loader.batch_sampler) == 0:
+
+                        if num_batches == max_batches - 1:
+                            for k, trainitem in self.training_items.items():
+                                trainitem.save_model()
+                            return True
+
+                        if only_steps==False and (mbs + 1) % len(data_loader.batch_sampler) == 0:
                             break
 
-                    if self.save_model_frequency > 0 and self.save_model_unit == 'epoch' and (epoch + 1) % self.save_model_frequency == 0:
-                        for k, trainitem in self.training_items.items():
-                            trainitem.save_model()
-
-
-
                 except StopIteration:
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_epoch_end()
@@ -1223,12 +1322,17 @@ class TrainingPlan(object):
                     PrintException()
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_training_end()
-
                 except Exception as e:
                     print(e)
                     PrintException()
                     for k, trainitem in self.training_items.items():
                         trainitem.do_on_training_end()
+                if self.save_model_frequency > 0 and self.save_model_unit == 'epoch' and (
+                        epoch + 1) % self.save_model_frequency == 0:
+                    for k, trainitem in self.training_items.items():
+                        trainitem.save_model()
+
+
         except KeyboardInterrupt:
             for k, trainitem in self.training_items.items():
                 trainitem.save_model()
@@ -1242,75 +1346,112 @@ class TrainingPlan(object):
         self.start_now(is_resume=True)
 
     def only_steps(self, num_steps, collect_data_inteval=1, keep_weights_history=False, keep_gradient_history=False):
-        self.execution_id = str(uuid.uuid4())[:8].__str__().replace('-', '')
-        # update callback
-        for item in self.training_items.values():
-            for callback in self.callbacks:
-                if callback not in item.callbacks:
-                    item.with_callbacks(callback)
-
-        data_loader = list(self._dataloaders.items())[0][1]
-        data_loader.minibatch_size = self.minibatch_size
-
-        tile_images_list = []
-
-        try:
-            # if self.lr_schedule is not None and trainingitem.is_optimizer_initialized()==True:
-            # don't print learning rate if print_progress_every unit is epoch
-            input = None
-            target = None
-            test_input = None
-            test_target = None
-            for mbs, iter_data in enumerate(data_loader):
-                if mbs < num_steps:
-                    iter_testdata = None
-                    if isinstance(data_loader, DataProviderV2):
-                        iter_testdata = data_loader.next_test()
-                    # input, target = Variable(input).to(self.device), Variable(target).to(self.device)
-
-                    for k, trainitem in self.training_items.items():
-                        trainitem.train_model(iter_data, iter_testdata, 0, mbs, 1, num_steps,
-                                              is_collect_data=mbs % collect_data_inteval == 0,
-                                              is_print_batch_progress=self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0,
-                                              is_print_epoch_progress=False,
-                                              log_gradients=keep_gradient_history and mbs % collect_data_inteval == 0,
-                                              log_weights=keep_weights_history and mbs % collect_data_inteval == 0,
-                                              accumulate_grads=False)
-                    if len(
-                            self.training_items.items()) > 2 and self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0:
-                        print('\n')
-
-                else:
-                    break
-            # if self.print_progress_on_epoch_end:
-            #     progress_bar(num_steps, num_steps, 'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.4e}'.format(
-            #         np.array(self.training_items[0].batch_loss_history['total_losses']).mean(),
-            #         ','.join(['{0}: {1:<8.3%}'.format(snake2camel(k), np.array(v).mean()) for k,
-            #         v in self.training_items[0].batch_metric_history.items()]),
-            #         optimizer.param_groups[0]['lr']))
-
-            batch_loss_history = [trainitem.batch_loss_history for k, trainitem in self.training_items.items()]
-            batch_metric_history = [trainitem.batch_metric_history for k, trainitem in self.training_items.items()]
-            loss_metric_curve(batch_loss_history, batch_metric_history, legend=self.training_names.value_list,
-                              sample_collected=self.training_items[0].sample_collect_history, max_iteration=num_steps,
-                              calculate_base='batch', save_path=self.tile_image_save_path,
-                              imshow=self.tile_image_imshow)
-
-
-        except StopIteration:
-            for k, trainitem in self.training_items.items():
-                trainitem.do_on_training_end()
-        except ValueError as ve:
-            for k, trainitem in self.training_items.items():
-                trainitem.do_on_training_end()
-            print(ve)
-            PrintException()
-        except Exception as e:
-            for k, trainitem in self.training_items.items():
-                trainitem.do_on_training_end()
-            print(e)
-            PrintException()
-
+        return self.start_now(collect_data_inteval=collect_data_inteval, is_resume=False,only_steps=True,max_batches=num_steps,keep_weights_history=keep_weights_history, keep_gradient_history=keep_gradient_history)
+        # self.execution_id = str(uuid.uuid4())[:8].__str__().replace('-','') if self.execution_id is None else self.execution_id
+        #
+        # # update callback
+        #
+        # for item in self.training_items.values():
+        #     for callback in self.callbacks:
+        #         if callback not in item.callbacks:
+        #             # private callback
+        #             if callback.is_shared == False:
+        #                 item.with_callbacks(copy.deepcopy(callback))
+        #             else:
+        #                 # shared callback
+        #                 item.with_callbacks(callback)
+        # # shared callbacks will access training plan dict instead of training_context
+        # for callback in self.callbacks:
+        #     if callback.is_shared == True:
+        #         callback.on_training_start(self.__dict__)
+        #
+        # data_loader = self._dataloaders.value_list[0]
+        # data_loader.minibatch_size = self.minibatch_size
+        # # generate data feed
+        #
+        # self.generate_datafeed(data_loader)
+        #
+        # try:
+        #
+        #     for mbs, return_data in enumerate(data_loader):
+        #         if mbs < num_steps:
+        #             iter_data = OrderedDict()
+        #             for i in range(len(data_loader.signature.key_list)):
+        #                 name = data_loader.signature.key_list[i]
+        #                 iter_data[name] = return_data[i]
+        #
+        #             # check weather need out-of-sample evaluation
+        #             need_out_sample_evaluation = False
+        #             if self.out_sample_evaluation_on_epoch_end == True and mbs == len(data_loader.batch_sampler) - 1:
+        #                 need_out_sample_evaluation = True
+        #             if self.out_sample_evaluation_unit == 'batch' and mbs > 0 and mbs % self.out_sample_evaluation_frequency == 0:
+        #                 need_out_sample_evaluation = True
+        #
+        #             iter_testdata = None
+        #             if isinstance(data_loader,
+        #                           DataProviderV2) and data_loader.testdata is not None and need_out_sample_evaluation:
+        #                 return_test = data_loader.next_test()
+        #                 if return_test is not None:
+        #                     iter_testdata = OrderedDict()
+        #                     for i in range(len(data_loader.signature.key_list)):
+        #                         name = data_loader.signature.key_list[i]
+        #                         iter_testdata[name] = return_test[i]
+        #
+        #             # input, target = Variable(input).to(self.device), Variable(target).to(self.device)
+        #
+        #             for trainitem_name, trainitem in zip(self.training_names.value_list,
+        #                                                  self.training_items.value_list):
+        #                 train_data = OrderedDict()
+        #                 test_data = None if iter_testdata is None else OrderedDict()
+        #                 for k, v in iter_data.items():
+        #                     train_data[k] = v.copy()
+        #
+        #                 if iter_testdata is not None:
+        #                     for k, v in iter_testdata.items():
+        #                         test_data[k] = v.copy()
+        #
+        #                 trainitem.training_context['model_name'] = trainitem_name
+        #                 trainitem.train_model(train_data,test_data, 0, mbs, 1, num_steps,
+        #                                       is_collect_data=mbs % collect_data_inteval == 0,
+        #                                       is_print_batch_progress=self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0,
+        #                                       is_print_epoch_progress=False,
+        #                                       log_gradients=keep_gradient_history and mbs % collect_data_inteval == 0,
+        #                                       log_weights=keep_weights_history and mbs % collect_data_inteval == 0,
+        #                                       accumulate_grads=False)
+        #             if len(self.training_items.items()) > 2 and self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0:
+        #                 print('\n')
+        #
+        #         else:
+        #             break
+        #     # if self.print_progress_on_epoch_end:
+        #     #     progress_bar(num_steps, num_steps, 'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.4e}'.format(
+        #     #         np.array(self.training_items[0].batch_loss_history['total_losses']).mean(),
+        #     #         ','.join(['{0}: {1:<8.3%}'.format(snake2camel(k), np.array(v).mean()) for k,
+        #     #         v in self.training_items[0].batch_metric_history.items()]),
+        #     #         optimizer.param_groups[0]['lr']))
+        #
+        #     batch_loss_history = [trainitem.batch_loss_history for k, trainitem in self.training_items.items()]
+        #     batch_metric_history = [trainitem.batch_metric_history for k, trainitem in self.training_items.items()]
+        #     loss_metric_curve(batch_loss_history, batch_metric_history, legend=self.training_names.value_list,
+        #                       sample_collected=self.training_items[0].sample_collect_history, max_iteration=num_steps,
+        #                       calculate_base='batch', save_path=self.tile_image_save_path,
+        #                       imshow=self.tile_image_imshow)
+        #
+        #
+        # except StopIteration:
+        #     for k, trainitem in self.training_items.items():
+        #         trainitem.do_on_training_end()
+        # except ValueError as ve:
+        #     for k, trainitem in self.training_items.items():
+        #         trainitem.do_on_training_end()
+        #     print(ve)
+        #     PrintException()
+        # except Exception as e:
+        #     for k, trainitem in self.training_items.items():
+        #         trainitem.do_on_training_end()
+        #     print(e)
+        #     PrintException()
+        #
 
 last_time = time.time()
 begin_time = last_time
